@@ -17,10 +17,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 
-
-# GUIOPT.py - scripts to control options windows mostly from GUIOPT winpack
-
-# GUIOPT:
+# GUIOPT.py - scripts to control options windows mostly from the GUIOPT winpack:
 # 0 - Main options window (peacock tail)
 # 1 - Video options window
 # 2 - msg win with 1 button
@@ -31,17 +28,15 @@
 # 8 - Feedback options window
 # 9 - Autopause options window
 
-
 ###################################################
 import GemRB
+import GUICommon
 import GUICommonWindows
+import GUISAVE
 from GUIDefines import *
-from GUICommon import CloseOtherWindow
-from GUICommon import GameWindow
-from GUISAVE import *
-from GUICommonWindows import *
- 
+
 ###################################################
+OptionsWindow = None
 SubOptionsWindow = None
 SubSubOptionsWindow = None
 GameOptionsWindow = None
@@ -49,13 +44,12 @@ OldOptionsWindow = None
 MoviesWindow = None
 KeysWindow = None
 HelpTextArea = None
-HelpTextArea2 = None
 
 LoadMsgWindow = None
 QuitMsgWindow = None
 hideflag = None
 
-
+###################################################
 def CloseOptionsWindow ():
 	global GameOptionsWindow, OptionsWindow
 	global OldOptionsWindow
@@ -69,74 +63,70 @@ def CloseOptionsWindow ():
 		OptionsWindow.Unload ()
 
 	GameOptionsWindow = None
-	SetSelectionChangeHandler (None)
+	GUICommonWindows.SetSelectionChangeHandler (None)
 	GemRB.SetVar ("OtherWindow", -1)
-	GameWindow.SetVisible(WINDOW_VISIBLE)
-	OptionsWindow = OldOptionsWindow
+	GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE)
+	GUICommonWindows.OptionsWindow = OldOptionsWindow
 	OldOptionsWindow = None
 	return
 
-def DummyWindow ():
-	return
-
 ###################################################
 def OpenOptionsWindow ():
 	"""Open main options window"""
 	global GameOptionsWindow, OptionsWindow
 	global OldOptionsWindow
 
-	if CloseOtherWindow(OpenOptionsWindow):
+	if GUICommon.CloseOtherWindow (OpenOptionsWindow):
 		CloseOptionsWindow()
 		return
 
 	#hideflag = GemRB.HideGUI ()
-	GameWindow.SetVisible(WINDOW_INVISIBLE)
-	SetSelectionChangeHandler (DummyWindow)
+	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE)
+	GUICommonWindows.SetSelectionChangeHandler (None)
 
 	GemRB.LoadWindowPack ("GUIOPT", 640, 480)
 	GameOptionsWindow = Window = GemRB.LoadWindow (2)
 	GemRB.SetVar ("OtherWindow", GameOptionsWindow.ID)
 	if OldOptionsWindow == None:
-		OldOptionsWindow = OptionsWindow
+		OldOptionsWindow = GUICommonWindows.OptionsWindow
 		OptionsWindow = GemRB.LoadWindow (0)
-		SetupMenuWindowControls (OptionsWindow, 0, "OpenOptionsWindow")
+		GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 0, OpenOptionsWindow)
 		OptionsWindow.SetFrame ()
 
-	# Load game
-	Button = Window.GetControl (5)
-	Button.SetText (13729)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenLoadMsgWindow")
-
-	# Save Game
-	Button = Window.GetControl (6)
-	Button.SetText (13730)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenSaveMsgWindow")
+	# Return to Game
+	Button = Window.GetControl (11)
+	Button.SetText (10308)
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenOptionsWindow)
 
 	# Quit Game
 	Button = Window.GetControl (10)
 	Button.SetText (13731)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenQuitMsgWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenQuitMsgWindow)
 
-	# Graphics
+	# Load Game
+	Button = Window.GetControl (5)
+	Button.SetText (13729)
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLoadMsgWindow)
+
+	# Save Game
+	Button = Window.GetControl (6)
+	Button.SetText (13730)
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenSaveMsgWindow)
+
+	# Video Options
 	Button = Window.GetControl (7)
 	Button.SetText (17162)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenVideoOptionsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenVideoOptionsWindow)
 
-	# Audio
+	# Audio Options
 	Button = Window.GetControl (8)
 	Button.SetText (17164)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenAudioOptionsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenAudioOptionsWindow)
 
-	# Gameplay
+	# Gameplay Options
 	Button = Window.GetControl (9)
 	Button.SetText (17165)
-	#Button.SetText (10308)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenGameplayOptionsWindow")
-
-	# Return to game
-	Button = Window.GetControl (11)
-	Button.SetText (10308)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenOptionsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenGameplayOptionsWindow)
 
 	# game version, e.g. v1.1.0000
 	Label = Window.GetControl (0x1000000b)
@@ -147,8 +137,8 @@ def OpenOptionsWindow ():
 	GUICommonWindows.PortraitWindow.SetVisible (WINDOW_VISIBLE)
 	return
 
-
 ###################################################
+
 def CloseVideoOptionsWindow ():
 	global SubOptionsWindow
 
@@ -179,6 +169,7 @@ def OpenVideoOptionsWindow ():
 	OptRadio ('BPP', Window, 5, 37, 'BitsPerPixel', 16)
 	OptRadio ('BPP', Window, 6, 37, 'BitsPerPixel', 24)
 	OptRadio ('BPP', Window, 7, 37, 'BitsPerPixel', 32)
+
 	OptCheckbox ('FullScreen', Window, 9, 38, 'Full Screen', 1)
 
 	OptCheckbox ('TransShadow', Window, 51, 50, 'Translucent Shadows', 1)
@@ -187,7 +178,7 @@ def OpenVideoOptionsWindow ():
 	OptCheckbox ('SoftStandBlt', Window, 42, 48, 'SoftBltFast' ,1)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
-
+	return
 
 def DisplayHelpFullScreen ():
 	HelpTextArea.SetText (18000)
@@ -240,8 +231,6 @@ def OpenAudioOptionsWindow ():
 
 	HelpTextArea = OptHelpText ('AudioOptions', Window, 14, 18040)
 
-	Window.DeleteControl(16)
-
 	OptDone ('AudioOptions', Window, 24)
 	OptCancel ('AudioOptions', Window, 25)
 	OptButton ('CharacterSounds', Window, 13, 17778)
@@ -289,7 +278,7 @@ def CloseCharacterSoundsWindow ():
 
 def OpenCharacterSoundsWindow ():
 	"""Open character sounds window"""
-	global SubSubOptionsWindow, HelpTextArea2
+	global SubSubOptionsWindow, HelpTextArea
 
 	if SubSubOptionsWindow:
 		if SubSubOptionsWindow:
@@ -298,7 +287,7 @@ def OpenCharacterSoundsWindow ():
 
 	SubSubOptionsWindow = Window = GemRB.LoadWindow (12)
 
-	HelpTextArea2 = OptHelpText ('CharacterSounds', Window, 16, 18041)
+	HelpTextArea = OptHelpText ('CharacterSounds', Window, 16, 18041)
 
 	OptDone ('CharacterSounds', Window, 24)
 	OptCancel ('CharacterSounds', Window, 25)
@@ -313,17 +302,33 @@ def OpenCharacterSoundsWindow ():
 	OptRadio ('SelectionSounds', Window, 59, 57, 'Selection Sounds Frequency', 2)
 	OptRadio ('SelectionSounds', Window, 60, 57, 'Selection Sounds Frequency', 3)
 
-	Window.ShowModal (MODAL_SHADOW_GRAY) 
+	Window.ShowModal (MODAL_SHADOW_GRAY)
+
+def DisplayHelpSubtitles ():
+	HelpTextArea.SetText (18015)
+
+def DisplayHelpAttackSounds ():
+	HelpTextArea.SetText (18013)
+
+def DisplayHelpFootsteps ():
+	HelpTextArea.SetText (18014)
+
+def DisplayHelpCommandSounds ():
+	HelpTextArea.SetText (18016)
+
+def DisplayHelpSelectionSounds ():
+	HelpTextArea.SetText (11352)
+
 ###################################################
 
 def CloseGameplayOptionsWindow ():
 	global SubOptionsWindow
- 
+
 	if SubOptionsWindow:
 		SubOptionsWindow.Unload()
 	SubOptionsWindow = None
 	return
-  
+
 def OpenGameplayOptionsWindow ():
 	"""Open gameplay options window"""
 	global SubOptionsWindow, HelpTextArea
@@ -348,13 +353,17 @@ def OpenGameplayOptionsWindow ():
 	OptCheckbox ('DitherAlways', Window, 14, 25, 'Always Dither', 1)
 	OptCheckbox ('Gore', Window, 19, 27, 'Gore', 1)
 	OptCheckbox ('Infravision', Window, 42, 44, 'Infravision', 1)
-	OptCheckbox ('Weather', Window, 47, 46, 'Weather', 1) 
+	OptCheckbox ('Weather', Window, 47, 46, 'Weather', 1)
+	if GUICommon.GameIsBG2():
+		OptCheckbox ('RestUntilHealed', Window, 50, 48, 'Heal Party on Rest', 1)
 
 	OptButton ('FeedbackOptions', Window, 5, 17163)
 	OptButton ('AutopauseOptions', Window, 6, 17166)
+	if GUICommon.GameIsBG2():
+		OptButton ('HotkeyOptions', Window, 51, 816)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
-
+	return
 
 def DisplayHelpTooltipDelay ():
 	HelpTextArea.SetText (18017)
@@ -377,16 +386,16 @@ def DisplayHelpGore ():
 	HelpTextArea.SetText (18023)
 
 def DisplayHelpInfravision ():
-	HelpTextArea.SetText (11797) 
+	HelpTextArea.SetText (11797)
 
 def DisplayHelpWeather ():
 	HelpTextArea.SetText (20619)
- 
-def DisplayHelpRestUntilHealed ():
-	HelpTextArea.SetText (2242) 
 
+def DisplayHelpRestUntilHealed ():
+	HelpTextArea.SetText (2242)
 
 ###################################################
+
 def CloseFeedbackOptionsWindow ():
 	global SubSubOptionsWindow
 
@@ -397,7 +406,7 @@ def CloseFeedbackOptionsWindow ():
 
 def OpenFeedbackOptionsWindow ():
 	"""Open feedback options window"""
-	global SubSubOptionsWindow, HelpTextArea2
+	global SubSubOptionsWindow, HelpTextArea
 
 	if SubSubOptionsWindow:
 		if SubSubOptionsWindow:
@@ -406,22 +415,23 @@ def OpenFeedbackOptionsWindow ():
 
 	SubSubOptionsWindow = Window = GemRB.LoadWindow (9)
 
-	HelpTextArea2 = OptHelpText ('FeedbackOptions', Window, 28, 18043)
+	HelpTextArea = OptHelpText ('FeedbackOptions', Window, 28, 18043)
 
 	OptDone ('FeedbackOptions', Window, 26)
 	OptCancel ('FeedbackOptions', Window, 27)
 
 	OptSlider ('MarkerFeedback', Window, 8, 'GUI Feedback Level', 1)
-	OptSlider ('LocatorFeedback', Window, 9, 'Locator Feedback Level', 1) 
+	OptSlider ('LocatorFeedback', Window, 9, 'Locator Feedback Level', 1)
+
 	OptCheckbox ('ToHitRolls', Window, 10, 32, 'Rolls', 1)
 	OptCheckbox ('CombatInfo', Window, 11, 33, 'Combat Info', 1)
 	OptCheckbox ('Actions', Window, 12, 34, 'Actions', 1)
 	OptCheckbox ('States', Window, 13, 35, 'State Changes', 1)
 	OptCheckbox ('Selection', Window, 14, 36, 'Selection Text', 1)
-	OptCheckbox ('Miscellaneous', Window, 15, 37, 'Miscellaneous Text', 1) 
+	OptCheckbox ('Miscellaneous', Window, 15, 37, 'Miscellaneous Text', 1)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
-
+	return
 
 def DisplayHelpMarkerFeedback ():
 	HelpTextArea.SetText (18024)
@@ -430,22 +440,22 @@ def DisplayHelpLocatorFeedback ():
 	HelpTextArea.SetText (18025)
 
 def DisplayHelpToHitRolls ():
-	HelpTextArea2.SetText (18026)
+	HelpTextArea.SetText (18026)
 
 def DisplayHelpCombatInfo ():
-	HelpTextArea2.SetText (18027)
+	HelpTextArea.SetText (18027)
 
 def DisplayHelpActions ():
-	HelpTextArea2.SetText (18028)
+	HelpTextArea.SetText (18028)
 
 def DisplayHelpStates ():
-	HelpTextArea2.SetText (18029)
+	HelpTextArea.SetText (18029)
 
 def DisplayHelpSelection ():
-	HelpTextArea2.SetText (18030)
+	HelpTextArea.SetText (18030)
 
 def DisplayHelpMiscellaneous ():
-	HelpTextArea2.SetText (18031)
+	HelpTextArea.SetText (18031)
 
 ###################################################
 
@@ -459,7 +469,7 @@ def CloseAutopauseOptionsWindow ():
 
 def OpenAutopauseOptionsWindow ():
 	"""Open autopause options window"""
-	global SubSubOptionsWindow, HelpTextArea2
+	global SubSubOptionsWindow, HelpTextArea
 
 	if SubSubOptionsWindow:
 		if SubSubOptionsWindow:
@@ -468,7 +478,7 @@ def OpenAutopauseOptionsWindow ():
 
 	SubSubOptionsWindow = Window = GemRB.LoadWindow (10)
 
-	HelpTextArea2 = OptHelpText ('AutopauseOptions', Window, 15, 18044)
+	HelpTextArea = OptHelpText ('AutopauseOptions', Window, 15, 18044)
 
 	OptDone ('AutopauseOptions', Window, 11)
 	OptCancel ('AutopauseOptions', Window, 14)
@@ -480,10 +490,15 @@ def OpenAutopauseOptionsWindow ():
 	OptCheckbox ('WeaponUnusable', Window, 5, 21, 'Auto Pause State', 16)
 	OptCheckbox ('TargetGone', Window, 13, 22, 'Auto Pause State', 32)
 	OptCheckbox ('EndOfRound', Window, 25, 24, 'Auto Pause State', 64)
-	OptCheckbox ('EnemySighted', Window, 26, 27, 'Auto Pause State', 128)
+	if Window.HasControl(26, IE_GUI_BUTTON):
+		OptCheckbox ('EnemySighted', Window, 26, 27, 'Auto Pause State', 128)
+	if GUICommon.GameIsBG2():
+		OptCheckbox ('SpellCast', Window, 34, 30, 'Auto Pause State', 256)
+		OptCheckbox ('TrapFound', Window, 31, 33, 'Auto Pause State', 512)
+		OptCheckbox ('CenterOnActor', Window, 31, 33, 'Auto Pause Center', 1)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
-
+	return
 
 def DisplayHelpCharacterHit ():
 	HelpTextArea.SetText (18032)
@@ -509,12 +524,20 @@ def DisplayHelpEndOfRound ():
 def DisplayHelpEnemySighted ():
 	HelpTextArea.SetText (23514)
 
+def DisplayHelpSpellCast ():
+	HelpTextArea.SetText (58171)
+
+def DisplayHelpTrapFound ():
+	HelpTextArea.SetText (31872)
+
+def DisplayHelpCenterOnActor ():
+	HelpTextArea.SetText (10571)
 
 ###################################################
 
 def OpenSaveMsgWindow ():
 	GemRB.SetVar("QuitAfterSave",0)
-	OpenSaveWindow ()
+	GUISAVE.OpenSaveWindow ()
 	#save the game without quitting
 	return
 
@@ -531,18 +554,19 @@ def OpenLoadMsgWindow ():
 	# Load
 	Button = Window.GetControl (0)
 	Button.SetText (15590)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "LoadGamePress")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LoadGamePress)
 
 	# Cancel
 	Button = Window.GetControl (1)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseLoadMsgWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseLoadMsgWindow)
 
 	# Loading a game will destroy ...
 	Text = Window.GetControl (3)
 	Text.SetText (19531)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
+	return
 
 def CloseLoadMsgWindow ():
 	global LoadMsgWindow
@@ -564,7 +588,8 @@ def LoadGamePress ():
 	GemRB.SetNextScript ("GUILOAD")
 	return
 
-def SaveGamePress():
+#save game AND quit
+def SaveGamePress ():
 	global QuitMsgWindow
 
 	if QuitMsgWindow:
@@ -573,10 +598,10 @@ def SaveGamePress():
 	#we need to set a state: quit after save
 	GemRB.SetVar("QuitAfterSave",1)
 	OpenOptionsWindow()
-	OpenSaveWindow ()
+	GUISAVE.OpenSaveWindow ()
 	return
 
-def QuitGamePress():
+def QuitGamePress ():
 	global QuitMsgWindow
 
 	if QuitMsgWindow:
@@ -600,21 +625,21 @@ def OpenQuitMsgWindow ():
 	# Save
 	Button = Window.GetControl (0)
 	Button.SetText (15589)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SaveGamePress")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SaveGamePress)
 
 	# Quit Game
 	Button = Window.GetControl (1)
 	Button.SetText (15417)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "QuitGamePress")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, QuitGamePress)
 
 	# Cancel
 	Button = Window.GetControl (2)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseQuitMsgWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseQuitMsgWindow)
 
 	# Do you wish to save the game ....
 	Text = Window.GetControl (3)
-	Text.SetText (16456) # or ??? - cannot be saved atm
+	Text.SetText (16456)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -626,8 +651,13 @@ def CloseQuitMsgWindow ():
 		QuitMsgWindow.Unload ()
 	QuitMsgWindow = None
 	OptionsWindow.SetVisible (WINDOW_VISIBLE)
+###################################################
+#TODO
+def OpenHotkeyOptionsWindow ():
 	return
 
+def CloseHotkeyOptionsWindow ():
+	return
 
 ###################################################
 # These functions help to setup controls found
@@ -637,18 +667,12 @@ def CloseQuitMsgWindow ():
 # These controls are usually made from an active
 # control (button, slider ...) and a label
 
+
 def OptSlider (name, window, slider_id, variable, value):
 	"""Standard slider for option windows"""
 	slider = window.GetControl (slider_id)
 	slider.SetVarAssoc (variable, value)
-	slider.SetEvent (IE_GUI_SLIDER_ON_CHANGE, "DisplayHelp" + name)
-
-	#label = window.GetControl (label_id)
-	#label.SetText (label_strref)
-	#label.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-	#label.SetState (IE_GUI_BUTTON_LOCKED)
-	#label.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, "DisplayHelp" + name)
-
+	slider.SetEvent (IE_GUI_SLIDER_ON_CHANGE, eval("DisplayHelp" + name))
 	return slider
 
 def OptRadio (name, window, button_id, label_id, variable, value):
@@ -656,7 +680,7 @@ def OptRadio (name, window, button_id, label_id, variable, value):
 
 	button = window.GetControl (button_id)
 	button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DisplayHelp" + name)
+	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, eval("DisplayHelp" + name))
 	button.SetVarAssoc (variable, value)
 
 	label = window.GetControl (label_id)
@@ -670,31 +694,33 @@ def OptCheckbox (name, window, button_id, label_id, variable, value):
 
 	button = window.GetControl (button_id)
 	button.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
-	button.SetState (IE_GUI_BUTTON_SELECTED)
-	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DisplayHelp" + name)
+	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, eval("DisplayHelp" + name))
 	button.SetVarAssoc (variable, value)
+
+	label = window.GetControl (label_id)
+	label.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
+	label.SetState (IE_GUI_BUTTON_LOCKED)
+
 	return button
 
 def OptButton (name, window, button_id, button_strref):
 	"""Standard subwindow button for option windows"""
 	button = window.GetControl (button_id)
-	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "Open%sWindow" %name)
-
+	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, eval("Open%sWindow" %name))
 	button.SetText (button_strref)
 
-
 def OptDone (name, window, button_id):
 	"""Standard `Done' button for option windows"""
 	button = window.GetControl (button_id)
 	button.SetText (11973) # Done
-	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "Close%sWindow" %name)
+	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, eval("Close%sWindow" %name))
 	button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 def OptCancel (name, window, button_id):
 	"""Standard `Cancel' button for option windows"""
 	button = window.GetControl (button_id)
 	button.SetText (13727) # Cancel
-	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "Close%sWindow" %name)
+	button.SetEvent (IE_GUI_BUTTON_ON_PRESS, eval("Close%sWindow" %name))
 	button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
 def OptHelpText (name, window, text_id, text_strref):
@@ -703,6 +729,5 @@ def OptHelpText (name, window, text_id, text_strref):
 	text.SetText (text_strref)
 	return text
 
-
 ###################################################
 # End of file GUIOPT.py
