@@ -21,16 +21,16 @@
 # GUIREC.py - scripts to control stats/records windows from the GUIREC winpack
 ###################################################
 import GemRB
-import GUICommonWindows
+import GUICommon
+import CommonTables
 from GUIDefines import *
 from ie_stats import *
 from ie_restype import *
-from GUICommon import *
-from LUCommon import CanLevelUp, GetNextLevelExp
-from GUICommonWindows import *
-from GUIWORLD import OpenReformPartyWindow
-from GUICG19 import NextSound
-
+import LUCommon
+import LevelUp
+import GUIWORLD
+import GUICG19
+import DualClass
 ###################################################
 RecordsWindow = None
 InformationWindow = None
@@ -46,6 +46,9 @@ ColorTable = None
 ColorIndex = None
 ScriptTextArea = None
 SelectedTextArea = None
+PortraitWindow = None
+OldPortraitWindow = None
+OldVoiceSet = None
 
 # the available sounds
 SoundSequence = [ 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', \
@@ -55,84 +58,92 @@ SoundIndex = 0
 
 ###################################################
 def OpenRecordsWindow ():
+	import GUICommonWindows
 	global RecordsWindow, OptionsWindow
-	global OldOptionsWindow
+	global OldOptionsWindow, PortraitWindow, OldPortraitWindow
 
-	if CloseOtherWindow (OpenRecordsWindow):
+	if GUICommon.CloseOtherWindow (OpenRecordsWindow):
 		if InformationWindow: OpenInformationWindow ()
 
 		if RecordsWindow:
 			RecordsWindow.Unload ()
 		if OptionsWindow:
 			OptionsWindow.Unload ()
+		if PortraitWindow:
+			PortraitWindow.Unload ()
 		RecordsWindow = None
 		GemRB.SetVar ("OtherWindow", -1)
-		GameWindow.SetVisible(WINDOW_VISIBLE)
+		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE)
 		GemRB.UnhideGUI ()
 		GUICommonWindows.OptionsWindow = OldOptionsWindow
 		OldOptionsWindow = None
-		SetSelectionChangeHandler (None)
+		GUICommonWindows.PortraitWindow = OldPortraitWindow
+		GUICommonWindows.UpdatePortraitWindow ()
+		OldPortraitWindow = None
+		GUICommonWindows.SetSelectionChangeHandler (None)
 		return
 
 	GemRB.HideGUI ()
-	GameWindow.SetVisible(WINDOW_INVISIBLE)
+	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE)
 
-	GemRB.LoadWindowPack ("GUIREC")
+	GemRB.LoadWindowPack ("GUIREC", 640, 480)
 	RecordsWindow = Window = GemRB.LoadWindow (2)
 	GemRB.SetVar ("OtherWindow", RecordsWindow.ID)
 	# saving the original portrait window
 	OldOptionsWindow = GUICommonWindows.OptionsWindow
 	OptionsWindow = GemRB.LoadWindow (0)
-	SetupMenuWindowControls (OptionsWindow, 0, "OpenRecordsWindow")
+	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 0, OpenRecordsWindow)
+	OldPortraitWindow = GUICommonWindows.PortraitWindow
+	PortraitWindow = GUICommonWindows.OpenPortraitWindow (0)
 	OptionsWindow.SetFrame ()
 
 	# dual class
 	Button = Window.GetControl (0)
 	Button.SetText (7174)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DualClassWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DualClass.DualClassWindow)
 
 	# levelup
 	Button = Window.GetControl (37)
 	Button.SetText (7175)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenLevelUpWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUp.OpenLevelUpWindow)
 
 	# information
 	Button = Window.GetControl (1)
 	Button.SetText (11946)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenInformationWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
 
 	# reform party
 	Button = Window.GetControl (51)
 	Button.SetText (16559)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenRecReformPartyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenRecReformPartyWindow)
 
 	# customize
 	Button = Window.GetControl (50)
 	Button.SetText (10645)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenCustomizeWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenCustomizeWindow)
 
 	# export
 	Button = Window.GetControl (36)
 	Button.SetText (13956)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenExportWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenExportWindow)
 
 ## 	# kit info
 ## 	Button = Window.GetControl (52)
 ## 	Button.SetText (61265)
-## 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "KitInfoWindow")
+## 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, KitInfoWindow)
 
-	SetSelectionChangeHandler (UpdateRecordsWindow)
+	GUICommonWindows.SetSelectionChangeHandler (UpdateRecordsWindow)
 	UpdateRecordsWindow ()
 
 	OptionsWindow.SetVisible (WINDOW_VISIBLE)
 	Window.SetVisible (WINDOW_VISIBLE)
-	GUICommonWindows.PortraitWindow.SetVisible (WINDOW_VISIBLE)
+	PortraitWindow.SetVisible (WINDOW_VISIBLE)
 	return
 
 #original returns to game before continuing...
 def OpenRecReformPartyWindow ():
 	OpenRecordsWindow ()
-	GemRB.SetTimedEvent ("OpenReformPartyWindow", 1)
+	GemRB.SetTimedEvent (GUIWORLD.OpenReformPartyWindow, 1)
 	return
 
 def UpdateRecordsWindow ():
@@ -154,14 +165,14 @@ def UpdateRecordsWindow ():
 
 	# dual-classable
 	Button = Window.GetControl (0)
-	if CanDualClass (pc):
+	if GUICommon.CanDualClass (pc):
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 	else:
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 
 	# levelup
 	Button = Window.GetControl (37)
-	if CanLevelUp (pc):
+	if LUCommon.CanLevelUp (pc):
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -239,12 +250,12 @@ def UpdateRecordsWindow ():
 	Label.SetTextColor (cchr[0], cchr[1], cchr[2])
 
 	# class
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	Label = Window.GetControl (0x10000030)
 	Label.SetText (ClassTitle)
 
 	# race
-	text = RaceTable.GetValue (RaceTable.FindValue (3, GemRB.GetPlayerStat (pc, IE_RACE)) ,
+	text = CommonTables.Races.GetValue (CommonTables.Races.FindValue (3, GemRB.GetPlayerStat (pc, IE_RACE)) ,
  0)
 
 	Label = Window.GetControl (0x1000000f)
@@ -293,6 +304,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	StateTable = GemRB.LoadTable ("statdesc")
 
 	GS = lambda s, pc=pc: GemRB.GetPlayerStat (pc, s)
+	GB = lambda s, pc=pc: GemRB.GetPlayerStat (pc, s, 1)
 	GA = lambda s, col, pc=pc: GemRB.GetAbilityBonus (s, col, GS (s) )
 
 	stats = []
@@ -302,16 +314,20 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# Next Level: <NEXTLEVEL>
 
 	# collecting tokens for stat overview
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	GemRB.SetToken ("CLASS", ClassTitle)
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	Class = ClassTable.FindValue (5, Class)
-	Class = ClassTable.GetRowName (Class)
-	Dual = IsDualClassed (pc, 1)
-	Multi = IsMultiClassed (pc, 1)
+	Class = CommonTables.Classes.FindValue (5, Class)
+	Class = CommonTables.Classes.GetRowName (Class)
+	Dual = GUICommon.IsDualClassed (pc, 1)
+	Multi = GUICommon.IsMultiClassed (pc, 1)
 	XP = GemRB.GetPlayerStat (pc, IE_XP)
 	LevelDrain = GS (IE_LEVELDRAIN)
 
+	if GS (IE_STATE_ID) & STATE_DEAD:
+		stats.append ( (11829,1,'c') ) # DEAD
+		stats.append (None)
+
 	if Multi[0] > 1: # we're multiclassed
 		print "\tMulticlassed"
 		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL), GemRB.GetPlayerStat (pc, IE_LEVEL2), GemRB.GetPlayerStat (pc, IE_LEVEL3)]
@@ -319,17 +335,17 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		stats.append ( (19721,1,'c') )
 		stats.append (None)
 		for i in range (Multi[0]):
-			ClassIndex = ClassTable.FindValue (5, Multi[i+1])
-			ClassTitle = GemRB.GetString (ClassTable.GetValue (ClassIndex, 2))
+			ClassIndex = CommonTables.Classes.FindValue (5, Multi[i+1])
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (ClassIndex, 2))
 			GemRB.SetToken ("CLASS", ClassTitle)
-			Class = ClassTable.GetRowName (ClassIndex)
+			Class = CommonTables.Classes.GetRowName (ClassIndex)
 			GemRB.SetToken ("LEVEL", str (Levels[i]+LevelDiff[i]-int(LevelDrain/Multi[0])) )
 			GemRB.SetToken ("EXPERIENCE", str (XP/Multi[0]) )
 			if LevelDrain:
 				stats.append ( (GemRB.GetString (19720),1,'d') )
 				stats.append ( (GemRB.GetString (57435),1,'d') ) # LEVEL DRAINED
 			else:
-				GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Levels[i]+LevelDiff[i], Class) )
+				GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Levels[i]+LevelDiff[i], Class) )
 				stats.append ( (GemRB.GetString (16480),"",'d') )
 			stats.append (None)
 			print "\t\tClass (Level):",Class,"(",Levels[i],")"
@@ -343,44 +359,44 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 		# the levels are stored in the class order (eg. FIGHTER_MAGE)
 		# the current active class does not matter!
-		if IsDualSwap (pc):
+		if GUICommon.IsDualSwap (pc):
 			Levels = [Levels[1], Levels[0], Levels[2]]
 
 		Levels[0] += LevelDiff[0]
 
-		ClassTitle = GemRB.GetString (ClassTable.GetValue (Dual[2], 2))
+		ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[2], 2))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[0]-LevelDrain))
-		Class = ClassTable.GetRowName (Dual[2])
+		Class = CommonTables.Classes.GetRowName (Dual[2])
 		XP2 = GemRB.GetPlayerStat (pc, IE_XP)
 		GemRB.SetToken ("EXPERIENCE", str (XP2) )
 		if LevelDrain:
 			stats.append ( (GemRB.GetString (19720),1,'d') )
 			stats.append ( (GemRB.GetString (57435),1,'d') ) # LEVEL DRAINED
 		else:
-			GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Levels[0], Class) )
+			GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Levels[0], Class) )
 			stats.append ( (GemRB.GetString (16480),"",'d') )
 		stats.append (None)
 		# the first class (shown second)
 		if Dual[0] == 1:
-			ClassTitle = GemRB.GetString (KitListTable.GetValue (Dual[1], 2))
+			ClassTitle = GemRB.GetString (CommonTables.KitList.GetValue (Dual[1], 2))
 		elif Dual[0] == 2:
-			ClassTitle = GemRB.GetString (ClassTable.GetValue (Dual[1], 2))
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[1], 2))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[1]) )
 
 		# the xp table contains only classes, so we have to determine the base class for kits
 		if Dual[0] == 2:
-			BaseClass = ClassTable.GetRowName (Dual[1])
+			BaseClass = CommonTables.Classes.GetRowName (Dual[1])
 		else:
-			BaseClass = GetKitIndex (pc)
-			BaseClass = KitListTable.GetValue (BaseClass, 7)
-			BaseClass = ClassTable.FindValue (5, BaseClass)
-			BaseClass = ClassTable.GetRowName (BaseClass)
+			BaseClass = GUICommon.GetKitIndex (pc)
+			BaseClass = CommonTables.KitList.GetValue (BaseClass, 7)
+			BaseClass = CommonTables.Classes.FindValue (5, BaseClass)
+			BaseClass = CommonTables.Classes.GetRowName (BaseClass)
 		# the first class' XP is discarded and set to the minimum level
 		# requirement, so if you don't dual class right after a levelup,
 		# the game would eat some of your XP
-		XP1 = NextLevelTable.GetValue (BaseClass, str (Levels[1]))
+		XP1 = CommonTables.NextLevel.GetValue (BaseClass, str (Levels[1]))
 		GemRB.SetToken ("EXPERIENCE", str (XP1) )
 
 		# inactive until the new class SURPASSES the former
@@ -399,7 +415,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 			stats.append ( (19720,1,'c') )
 			stats.append ( (57435,1,'c') ) # LEVEL DRAINED
 		else:
-			GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Level, Class) )
+			GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Level, Class) )
 			stats.append ( (16480,1,'c') )
 		stats.append (None)
 		print "\t\tClass (Level):",Class,"(",Level,")"
@@ -451,7 +467,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 	#this hack only displays LOH if we know the spell
 	#TODO: the core should just not set LOH if the paladin can't learn it
-	if (HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
+	if (GUICommon.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
 		stats.append ( (12127, GS (IE_LAYONHANDSAMOUNT), '') )
 
 	#script
@@ -462,15 +478,15 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# 17379 Saving throws
 	stats.append (17379)
 	# 17380 Paralyze/Poison/Death
-	stats.append ( (17380, GS (IE_SAVEVSDEATH), '0') )
+	stats.append ( (17380, IE_SAVEVSDEATH, 's') )
 	# 17381 Rod/Staff/Wand
-	stats.append ( (17381, GS (IE_SAVEVSWANDS), '0') )
+	stats.append ( (17381, IE_SAVEVSWANDS, 's') )
 	# 17382 Petrify/Polymorph
-	stats.append ( (17382, GS (IE_SAVEVSPOLY), '0') )
+	stats.append ( (17382, IE_SAVEVSPOLY, 's') )
 	# 17383 Breath weapon
-	stats.append ( (17383, GS (IE_SAVEVSBREATH), '0') )
+	stats.append ( (17383, IE_SAVEVSBREATH, 's') )
 	# 17384 Spells
-	stats.append ( (17384, GS (IE_SAVEVSSPELL), '0') )
+	stats.append ( (17384, IE_SAVEVSSPELL, 's') )
 	stats.append (None)
 
 	# 9466 Weapon proficiencies
@@ -514,7 +530,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# 10341 Reaction adjustment
 	stats.append ( (10341, GA (IE_DEX,0), '0') )
 	# 10342 CON HP Bonus/Level
-	stats.append ( (10342, GA (IE_CON,0), '0') )
+	stats.append ( (10342, GA (IE_CON,0), 'p') )
 	# 10343 Chance To Learn spell
 	if GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_WIZARD, 0, 0)>0:
 		stats.append ( (10343, GA (IE_INT,0), '%' ) )
@@ -543,6 +559,18 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 				continue
 			if type == '+': #pluses
 				res.append ("[capital=0]"+GemRB.GetString (strref) + ' '+ '+' * val)
+			elif type == 'p': #a plus prefix if positive
+				if val > 0:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ' +' + str (val) )
+				else:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ' ' + str (val) )
+			elif type == 's': #both base and (modified) stat, but only if they differ
+				base = str (GB (val))
+				stat = str (GS (val))
+				if base == stat:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ': ' + base)
+				else:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ': ' + base + " (" + stat + ")")
 			elif type == 'x': #x character before value
 				res.append ("[capital=0]"+GemRB.GetString (strref) +': x' + str (val) )
 			elif type == 'a': #value (portrait icon) + string
@@ -589,12 +617,12 @@ def OpenInformationWindow ():
 	# Biography
 	Button = Window.GetControl (26)
 	Button.SetText (18003)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenBiographyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyWindow)
 
 	# Done
 	Button = Window.GetControl (24)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseInformationWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseInformationWindow)
 
 	TotalPartyExp = 0
 	ChapterPartyExp = 0
@@ -614,7 +642,7 @@ def OpenInformationWindow ():
 	Label = Window.GetControl (0x10000000)
 	Label.SetText (GemRB.GetPlayerName (pc, 1))
 	# class
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	Label = Window.GetControl (0x10000018)
 	Label.SetText (ClassTitle)
 
@@ -654,15 +682,16 @@ def OpenInformationWindow ():
 	#are they used?
 	Label.SetText (stat['FavouriteWeapon'])
 
-	#total xp
-	Label = Window.GetControl (0x1000000f)
+	#total party xp
+	Label = Window.GetControl (0x10000013)
 	if TotalPartyExp != 0:
 		PartyExp = int ((stat['KillsTotalXP'] * 100) / TotalPartyExp)
 		Label.SetText (str (PartyExp) + '%')
 	else:
 		Label.SetText ("0%")
 
-	Label = Window.GetControl (0x10000013)
+	# chapter party xp
+	Label = Window.GetControl (0x1000000f)
 	if ChapterPartyExp != 0:
 		PartyExp = int ((stat['KillsChapterXP'] * 100) / ChapterPartyExp)
 		Label.SetText (str (PartyExp) + '%')
@@ -670,14 +699,15 @@ def OpenInformationWindow ():
 		Label.SetText ("0%")
 
 	#total xp
-	Label = Window.GetControl (0x10000010)
+	Label = Window.GetControl (0x10000014)
 	if TotalCount != 0:
 		PartyExp = int ((stat['KillsTotalCount'] * 100) / TotalCount)
 		Label.SetText (str (PartyExp) + '%')
 	else:
 		Label.SetText ("0%")
 
-	Label = Window.GetControl (0x10000014)
+	# chapter xp
+	Label = Window.GetControl (0x10000010)
 	if ChapterCount != 0:
 		PartyExp = int ((stat['KillsChapterCount'] * 100) / ChapterCount)
 		Label.SetText (str (PartyExp) + '%')
@@ -699,6 +729,7 @@ def OpenInformationWindow ():
 	return
 
 def CloseInformationWindow ():
+	import GUICommonWindows
 	global InformationWindow
 
 	if InformationWindow:
@@ -706,7 +737,7 @@ def CloseInformationWindow ():
 	InformationWindow = None
 	OptionsWindow.SetVisible (WINDOW_VISIBLE)
 	RecordsWindow.SetVisible (WINDOW_VISIBLE)
-	GUICommonWindows.PortraitWindow.SetVisible (WINDOW_VISIBLE)
+	PortraitWindow.SetVisible (WINDOW_VISIBLE)
 	return
 
 def OpenBiographyWindow ():
@@ -725,7 +756,7 @@ def OpenBiographyWindow ():
 	# Done
 	Button = Window.GetControl (2)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseBiographyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseBiographyWindow)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -760,9 +791,9 @@ def OpenExportWindow ():
 
 	NameField = ExportWindow.GetControl (6)
 
-	ExportDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ExportDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ExportCancelPress")
-	NameField.SetEvent (IE_GUI_EDIT_ON_CHANGE, "ExportEditChanged")
+	ExportDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportCancelPress)
+	NameField.SetEvent (IE_GUI_EDIT_ON_CHANGE, ExportEditChanged)
 	ExportWindow.ShowModal (MODAL_SHADOW_GRAY)
 	NameField.SetStatus (IE_GUI_CONTROL_FOCUSED)
 	return
@@ -836,13 +867,13 @@ def OpenCustomizeWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	PortraitSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenPortraitSelectWindow")
-	SoundButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenSoundWindow")
-	ColorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenColorWindow")
-	ScriptButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenScriptWindow")
-	#BiographyButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenBiographyEditWindow")
-	CustomizeDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CustomizeDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CustomizeCancelPress")
+	PortraitSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenPortraitSelectWindow)
+	SoundButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenSoundWindow)
+	ColorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenColorWindow)
+	ScriptButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenScriptWindow)
+	#BiographyButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyEditWindow)
+	CustomizeDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomizeDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomizeCancelPress)
 
 	CustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -894,11 +925,11 @@ def OpenPortraitSelectWindow ():
 	CustomPortraitButton = SubCustomizeWindow.GetControl (5)
 	CustomPortraitButton.SetText (17545)
 
-	LeftButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitLeftPress")
-	RightButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitRightPress")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
-	CustomPortraitButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenCustomPortraitWindow")
+	LeftButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitLeftPress)
+	RightButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitRightPress)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	CustomPortraitButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenCustomPortraitWindow)
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	while True:
@@ -972,19 +1003,19 @@ def OpenCustomPortraitWindow ():
 	# Portrait List Large
 	PortraitList1 = SubSubCustomizeWindow.GetControl (2)
 	RowCount1 = PortraitList1.GetPortraits (0)
-	PortraitList1.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "LargeCustomPortrait")
+	PortraitList1.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, LargeCustomPortrait)
 	GemRB.SetVar ("Row1", RowCount1)
 	PortraitList1.SetVarAssoc ("Row1",RowCount1)
 
 	# Portrait List Small
 	PortraitList2 = SubSubCustomizeWindow.GetControl (3)
 	RowCount2 = PortraitList2.GetPortraits (1)
-	PortraitList2.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "SmallCustomPortrait")
+	PortraitList2.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, SmallCustomPortrait)
 	GemRB.SetVar ("Row2", RowCount2)
 	PortraitList2.SetVarAssoc ("Row2",RowCount2)
 
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CustomPortraitDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubSubCustomizeWindow")
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomPortraitDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubSubCustomizeWindow)
 
 	SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1048,16 +1079,19 @@ def OpenSoundWindow ():
 	global SubCustomizeWindow
 	global VoiceList
 	global Gender
+	global OldVoiceSet
 
+	pc = GemRB.GameGetSelectedPCSingle ()
+	OldVoiceSet = GemRB.GetPlayerSound (pc)
 	SubCustomizeWindow = GemRB.LoadWindow (20)
 
 	VoiceList = SubCustomizeWindow.GetControl (5)
 	VoiceList.SetFlags (IE_GUI_TEXTAREA_SELECTABLE)
-	pc = GemRB.GameGetSelectedPCSingle ()
 	Gender = GemRB.GetPlayerStat (pc, IE_SEX, 1)
 
 	VoiceList.SetVarAssoc ("Selected", 0)
-	RowCount=VoiceList.GetCharSounds()
+	VoiceList.GetCharSounds()
+	VoiceList.SelectText (OldVoiceSet)
 
 	PlayButton = SubCustomizeWindow.GetControl (7)
 	PlayButton.SetText (17318)
@@ -1073,13 +1107,19 @@ def OpenSoundWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	PlayButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PlaySoundPressed")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneSoundWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	PlayButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PlaySoundPressed)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneSoundWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSoundWindow)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
+def CloseSoundWindow ():
+	pc = GemRB.GameGetSelectedPCSingle ()
+	GemRB.SetPlayerSound (pc, OldVoiceSet)
+	CloseSubCustomizeWindow ()
+	return
+
 def DoneSoundWindow ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	CharSound = VoiceList.QueryText ()
@@ -1092,9 +1132,21 @@ def PlaySoundPressed():
 	global CharSoundWindow, SoundIndex, SoundSequence
 
 	CharSound = VoiceList.QueryText ()
+	tmp = SoundIndex
 	while (not GemRB.HasResource (CharSound + SoundSequence[SoundIndex], RES_WAV)):
 		NextSound()
-	GemRB.PlaySound (CharSound + SoundSequence[SoundIndex], 0, 0, 4)
+		if SoundIndex == tmp:
+			break
+	else:
+		NextSound()
+	GemRB.PlaySound (CharSound + SoundSequence[SoundIndex], 0, 0, 5)
+	return
+
+def NextSound():
+	global SoundIndex, SoundSequence
+	SoundIndex += 1
+	if SoundIndex >= len(SoundSequence):
+		SoundIndex = 0
 	return
 
 def OpenColorWindow ():
@@ -1128,12 +1180,12 @@ def OpenColorWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	HairButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetHairColor")
-	SkinButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetSkinColor")
-	MajorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetMajorColor")
-	MinorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetMinorColor")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneColorWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	HairButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetHairColor)
+	SkinButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetSkinColor)
+	MajorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMajorColor)
+	MinorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMinorColor)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneColorWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
 	UpdatePaperDoll ()
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
@@ -1157,7 +1209,7 @@ def UpdatePaperDoll ():
 	Color5 = GemRB.GetPlayerStat (pc, IE_LEATHER_COLOR)
 	Color6 = GemRB.GetPlayerStat (pc, IE_ARMOR_COLOR)
 	HairButton.SetBAM ("COLGRAD", 0, 0, HairColor&0xff)
-	PortraitButton.SetPLT (GetActorPaperDoll (pc),
+	PortraitButton.SetPLT (GUICommon.GetActorPaperDoll (pc),
 		Color1, MinorColor, MajorColor, SkinColor, Color5, Color6, HairColor, 0, 0)
 	return
 
@@ -1202,8 +1254,8 @@ def OpenColorPicker ():
 	GemRB.SetVar ("Selected",-1)
 	for i in range (1,35):
 		Button = SubSubCustomizeWindow.GetControl (i)
-		Button.SetState (IE_GUI_BUTTON_DISABLED)
-		Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
+		Button.SetState (IE_GUI_BUTTON_LOCKED)
+		Button.SetFlags (IE_GUI_BUTTON_PICTURE,OP_OR)
 
 	#Selected = -1
 	for i in range (34):
@@ -1217,7 +1269,7 @@ def OpenColorPicker ():
 			#Selected = i
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc("Selected",i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DonePress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DonePress)
 
 	SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1271,9 +1323,9 @@ def OpenScriptWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneScriptWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
-	ScriptTextArea.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "UpdateScriptSelection")
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneScriptWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	ScriptTextArea.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, UpdateScriptSelection)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1343,10 +1395,10 @@ def OpenBiographyEditWindow ():
 	TextArea.SetBufferLength (65535)
 	TextArea.SetText (BioStrRef)
 
-	ClearButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ClearBiography")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneBiographyWindow")
-	RevertButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "RevertBiography")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	ClearButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ClearBiography)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneBiographyWindow)
+	RevertButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, RevertBiography)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
