@@ -26,14 +26,13 @@ from GUIDefines import *
 from ie_stats import *
 from ie_slots import *
 import GemRB
-from GUICommon import CloseOtherWindow, ClassTable, SetEncumbranceLabels
-from GUICommonWindows import GetActorPortrait, SetSelectionChangeHandler
-
+import GUICommon
+import CommonTables
+import GUICommonWindows
+import InventoryCommon
 
 InventoryWindow = None
-ItemInfoWindow = None
 ItemAmountWindow = None
-ItemIdentifyWindow = None
 OverSlot = None
 
 ItemHash = {}
@@ -64,14 +63,14 @@ def OpenInventoryWindow ():
 
 	AvSlotsTable = GemRB.LoadTable ('avslots')
 
-	if CloseOtherWindow (OpenInventoryWindow):
+	if GUICommon.CloseOtherWindow (OpenInventoryWindow):
 		GemRB.HideGUI ()
 		if InventoryWindow:
 			InventoryWindow.Unload ()
 		InventoryWindow = None
 		GemRB.SetVar ("OtherWindow", -1)
 		GemRB.SetVar ("MessageLabel", -1)
-		SetSelectionChangeHandler (None)
+		GUICommonWindows.SetSelectionChangeHandler (None)
 		GemRB.UnhideGUI ()
 		return
 
@@ -89,8 +88,8 @@ def OpenInventoryWindow ():
 		Button.SetVarAssoc ("ItemButton", i)
 		Button.SetBorder (0,0,0,0,0,128,128,255,64,0,1)
 		Button.SetBorder (1,0,0,0,0,255,128,128,64,0,1)
-		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, "MouseEnterSlot")
-		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, "MouseLeaveSlot")
+		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, MouseEnterSlot)
+		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, MouseLeaveSlot)
 
 	# Ground Item
 	for i in range (10):
@@ -100,11 +99,11 @@ def OpenInventoryWindow ():
 		Button.SetFont ("NUMBER2")
 		Button.SetBorder (0,0,0,0,0,128,128,255,64,0,1)
 		Button.SetBorder (1,0,0,0,0,255,128,128,64,0,1)
-		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, "MouseEnterGround")
-		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, "MouseLeaveGround")
+		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, InventoryCommon.MouseEnterGround)
+		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, InventoryCommon.MouseLeaveGround)
 
 	ScrollBar = Window.GetControl (45)
-	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, "RefreshInventoryWindow")
+	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, RefreshInventoryWindow)
 
 	for i in range (57, 64):
 		Label = Window.GetControl (0x10000000 + i)
@@ -114,7 +113,7 @@ def OpenInventoryWindow ():
 	Button = Window.GetControl (44)
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
 	Button.SetFlags(IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)
-	Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnAutoEquip")
+	Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, OnAutoEquip)
 
 	# encumbrance
 	Button = Window.GetControl (46)
@@ -139,7 +138,7 @@ def OpenInventoryWindow ():
 	Label.SetTextColor(255, 255, 255)
 	Label.SetText ("")
 	GemRB.SetVar ("TopIndex", 0)
-	SetSelectionChangeHandler (UpdateInventoryWindow)
+	GUICommonWindows.SetSelectionChangeHandler (UpdateInventoryWindow)
 	UpdateInventoryWindow ()
 
 	GemRB.UnhideGUI()
@@ -151,7 +150,7 @@ def UpdateInventoryWindow ():
 
 	Window = InventoryWindow
 
-	GemRB.RunEventHandler("UpdateAnimation")
+	GUICommonWindows.UpdateAnimation ()
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 
@@ -185,9 +184,9 @@ def RefreshInventoryWindow ():
 
 	# portrait
 	Button = Window.GetControl (44)
-	Button.SetPicture (GetActorPortrait (pc, 'INVENTORY'))
+	Button.SetPicture (GUICommonWindows.GetActorPortrait (pc, 'INVENTORY'))
 
-	SetEncumbranceLabels (Window, 46, None, pc)
+	GUICommon.SetEncumbranceLabels (Window, 46, None, pc, True)
 
 	# armor class
 	ac = GemRB.GetPlayerStat (pc, IE_ARMORCLASS)
@@ -210,11 +209,17 @@ def RefreshInventoryWindow ():
 	Label.SetText (str (GemRB.GameGetPartyGold ()))
 
 	# class
-	text = ClassTable.GetValue (GemRB.GetPlayerStat (pc, IE_CLASS) - 1, 0)
+	text = CommonTables.Classes.GetValue (GemRB.GetPlayerStat (pc, IE_CLASS) - 1, 0)
 
 	Label = Window.GetControl (0x1000003f)
 	Label.SetText (text)
 
+	held = GemRB.GetPlayerStat (pc, IE_HELD) + GemRB.GetPlayerStat (pc, IE_CASTERHOLD)
+	if held == 0 and GemRB.GetPlayerStat (pc, IE_STATE_ID) & STATE_DEAD == 0:
+		Window.SetVisible (WINDOW_VISIBLE)
+	else:
+		Window.SetVisible (WINDOW_GRAYED)
+
 	# update ground inventory slots
 	Container = GemRB.GetContainer(pc, 1)
 	TopIndex = GemRB.GetVar ("TopIndex")
@@ -224,7 +229,7 @@ def RefreshInventoryWindow ():
 			Button.SetState (IE_GUI_BUTTON_SECOND)
 		else:
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnDragItemGround")
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, InventoryCommon.OnDragItemGround)
 		Slot = GemRB.GetContainerItem (pc, i+TopIndex)
 		if Slot != None:
 			item = GemRB.GetItem (Slot['ItemResRef'])
@@ -238,17 +243,17 @@ def RefreshInventoryWindow ():
 			else:
 				Button.SetTooltip (item["ItemNameIdentified"])
 				Button.EnableBorder (0, 0)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OnDragItemGround")
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "OpenGroundItemInfoWindow")
-			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "OpenGroundItemAmountWindow")
+			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, InventoryCommon.OnDragItemGround)
+			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InventoryCommon.OpenGroundItemInfoWindow)
+			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, None) #TODO: implement OpenGroundItemAmountWindow
 
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
 			Button.SetTooltip (4273)
 			Button.EnableBorder (0, 0)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "")
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "")
-			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "")
+			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
+			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, None)
  	return
 
 def UpdateSlot (pc, i):
@@ -304,10 +309,10 @@ def UpdateSlot (pc, i):
 			Button.EnableBorder (1, 1)
 
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OnDragItem")
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "OpenItemInfoWindow")
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "OpenItemAmountWindow")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnDragItem")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnDragItem)
+		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InventoryCommon.OpenItemInfoWindow)
+		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, OpenItemAmountWindow)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, OnDragItem)
 	else:
 		Button.SetItemIcon ('')
 		Button.SetText ('')
@@ -315,19 +320,19 @@ def UpdateSlot (pc, i):
 		if slot_list[i]>=0:
 			Button.SetSprites (SlotType["ResRef"], 0, 0, 1, 2, 3)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-			Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnDragItem")
+			Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, OnDragItem)
 			Button.SetTooltip (SlotType["Tip"])
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
-			Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, '')
+			Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, None)
 			Button.SetTooltip ('')
 			itemname = ""
 
 		Button.EnableBorder (0, 0)
 		Button.EnableBorder (1, 0)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "")
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "")
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
+		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, None)
 
 	if OverSlot == slot-1:
 		if GemRB.CanUseItemType (SlotType["Type"], itemname):
@@ -344,21 +349,6 @@ def UpdateSlot (pc, i):
 			Button.SetState (IE_GUI_BUTTON_THIRD)
 	return
 
-def OnDragItemGround ():
-	pc = GemRB.GameGetSelectedPCSingle ()
-
-	slot = GemRB.GetVar ("GroundItemButton") + GemRB.GetVar ("TopIndex")
-	if not GemRB.IsDraggingItem ():
-		slot_item = GemRB.GetContainerItem (pc, slot)
-		item = GemRB.GetItem (slot_item["ItemResRef"])
-		GemRB.DragItem (pc, slot, item["ItemIcon"], 0, 1) #container
-		GemRB.PlaySound (item["DescIcon"])
-	else:
-		GemRB.DropDraggedItem (pc, -2) #dropping on ground
-
-	UpdateInventoryWindow ()
-	return
-
 def OnAutoEquip ():
 	if not GemRB.IsDraggingItem ():
 		return
@@ -397,13 +387,6 @@ def OnDragItem ():
 
 	UpdateInventoryWindow ()
 
-
-def OnDropItemToPC ():
-	pc = GemRB.GetVar ("PressedPortrait") + 1
-	GemRB.DropDraggedItem (pc, -3)
-	UpdateInventoryWindow ()
-	return
-
 def DragItemAmount ():
 	"""Drag a split item."""
 
@@ -452,13 +435,13 @@ def OpenItemAmountWindow ():
 	# Done
 	Button = Window.GetControl (2)
 	Button.SetText (1403)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DragItemAmount")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DragItemAmount)
 	Button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 	# Cancel
 	Button = Window.GetControl (1)
 	Button.SetText (4196)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenItemAmountWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenItemAmountWindow)
 	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
 	# 0 bmp
@@ -481,9 +464,8 @@ def LeftItemScroll ():
 			tmp = 43
 
 	GemRB.SetVar ('ItemButton', tmp)
-	#one for close, one for reopen
-	CloseItemInfoWindow ()
-	OpenItemInfoWindow ()
+	InventoryCommon.CloseItemInfoWindow ()
+	InventoryCommon.OpenItemInfoWindow ()
 	return
 
 def RightItemScroll ():
@@ -497,245 +479,8 @@ def RightItemScroll ():
 			tmp = 0
 
 	GemRB.SetVar ('ItemButton', tmp)
-	#one for close, one for reopen
-	CloseItemInfoWindow ()
-	OpenItemInfoWindow ()
-	return
-
-def DrinkItemWindow ():
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
-	# the drink item header is always the first
-	GemRB.UseItem (pc, slot, 0)
-	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
-	return
-
-def ReadItemWindow ():
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
-	# the learn scroll header is always the second
-	GemRB.UseItem (pc, slot, 1)
-	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
-	return
-
-def OpenItemWindow ():
-	#close inventory
-	GemRB.SetVar ("Inventory", 1)
-	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
-	OpenInventoryWindow ()
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
-	slot_item = GemRB.GetSlotItem (pc, slot)
-	ResRef = slot_item['ItemResRef']
-	#the store will have to reopen the inventory
-	GemRB.EnterStore (ResRef)
-	return
-
-def DialogItemWindow ():
-
-	CloseItemInfoWindow ()
-	OpenInventoryWindow ()
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot, slot_item = ItemHash[GemRB.GetVar ('ItemButton')]
-	ResRef = slot_item['ItemResRef']
-	item = GemRB.GetItem (ResRef)
-	dialog=item['Dialog']
-	GemRB.ExecuteString ("StartDialog(\""+dialog+"\",Myself)", pc)
-	return
-
-def IdentifyUseSpell ():
-	global ItemIdentifyWindow
-
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
-	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
-	GemRB.HasSpecialSpell (pc, 1, 1)
-	GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-	CloseIdentifyItemWindow()
-	return
-
-def IdentifyUseScroll ():
-	global ItemIdentifyWindow
-
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
-	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
-	if GemRB.HasSpecialItem (pc, 1, 1):
-		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-	CloseIdentifyItemWindow()
-	return
-
-def CloseIdentifyItemWindow ():
-	global ItemIdentifyWindow
-
-	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
-	ItemIdentifyWindow = None
-	return
-
-def IdentifyItemWindow ():
-	global ItemIdentifyWindow
-
-	pc = GemRB.GameGetSelectedPCSingle ()
-
-	ItemIdentifyWindow = Window = GemRB.LoadWindow (9)
-
-	# how would you like to identify ....
-	Text = Window.GetControl (3)
-	Text.SetText (4258)
-
-	# Spell
-	Button = Window.GetControl (0)
-	Button.SetText (4259)
-	if not GemRB.HasSpecialSpell (pc, 1, 0):
-		Button.SetState (IE_GUI_BUTTON_DISABLED)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "IdentifyUseSpell")
-
-	# Scroll
-	Button = Window.GetControl (1)
-	Button.SetText (4260)
-	if not GemRB.HasSpecialItem (pc, 1, 0):
-		Button.SetState (IE_GUI_BUTTON_DISABLED)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "IdentifyUseScroll")
-
-	# Cancel
-	Button = Window.GetControl (2)
-	Button.SetText (4196)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseIdentifyItemWindow")
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-
-	Window.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-def CloseItemInfoWindow ():
-	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
-	UpdateInventoryWindow()
-	return
-
-def DisplayItem (itemresref, type):
-	global ItemInfoWindow
-
-	item = GemRB.GetItem (itemresref)
-	ItemInfoWindow = Window = GemRB.LoadWindow (5)
-
-	# Done
-	Button = Window.GetControl (4)
-	Button.SetText (1403)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseItemInfoWindow")
-
-	# Identify
-	Button = Window.GetControl (8)
-	if type&2:
-		Button.SetText (4256)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "IdentifyItemWindow")
-	else:
-		Button.SetState (IE_GUI_BUTTON_LOCKED)
-		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-
-	# Talk to Item/Use
-	Button = Window.GetControl (9)
-	drink = (type&1) and (item["Function"]&1)
-	read = (type&1) and (item["Function"]&2)
-	container = (type&1) and (item["Function"]&4)
-	dialog = (type&1) and (item["Dialog"]!="")
-
-	Button.SetState (IE_GUI_BUTTON_ENABLED)
-	Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
-	if drink:
-		#no drink/food in PST, this is the closest strref in stock pst
-		Button.SetText (4251)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DrinkItemWindow")
-	elif read:
-		Button.SetText (4252)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ReadItemWindow")
-	elif container:
-		#no containers in PST, this is the closest strref in stock pst
-		Button.SetText (52399)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenItemWindow")
-	elif dialog:
-		Button.SetText (4254)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DialogItemWindow")
-	else:
-		Button.SetText ('')
-		Button.SetState (IE_GUI_BUTTON_LOCKED)
-		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-
-	Label = Window.GetControl (0x10000000)
-	if type&2:
-		Label.SetText (GemRB.GetString (item['ItemName']).upper())
-	else:
-		Label.SetText (GemRB.GetString (item['ItemNameIdentified']).upper())
-
-	Icon = Window.GetControl (2)
-	Icon.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-	Icon.SetItemIcon (itemresref)
-
-	Text = Window.GetControl (5)
-	if type&2:
-		Text.SetText (item['ItemDesc'])
-	else:
-		Text.SetText (item['ItemDescIdentified'])
-
-	IdText = Window.GetControl (0x1000000b)
-	if type&2:
-		# 4279 - This item has not been identified
-		IdText.SetText (4279)
-	else:
-		IdText.SetText ('')
-
-	Icon = Window.GetControl (7)
-	Icon.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-	Icon.SetItemIcon (itemresref, 1)
-
-	#left scroll
-	Button = Window.GetControl (13)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "LeftItemScroll")
-
-	#right scroll
-	Button = Window.GetControl (14)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "RightItemScroll")
-
-	Window.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-def OpenItemInfoWindow ():
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot, slot_item = ItemHash[GemRB.GetVar ('ItemButton')]
-	ResRef = slot_item['ItemResRef']
-	item = GemRB.GetItem (slot_item["ItemResRef"])
-
-	#auto identify when lore is high enough
-	if item["LoreToID"]<=GemRB.GetPlayerStat (pc, IE_LORE):
-		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-		slot_item["Flags"] |= IE_INV_ITEM_IDENTIFIED
-		UpdateInventoryWindow ()
-
-	if slot_item["Flags"] & IE_INV_ITEM_IDENTIFIED:
-		value = 1
-	else:
-		value = 3
-	DisplayItem (slot_item["ItemResRef"], value)
-	return
-
-def OpenGroundItemInfoWindow ():
-	global ItemInfoWindow
-
-	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("TopIndex") + GemRB.GetVar ("GroundItemButton")
-	slot_item = GemRB.GetContainerItem (pc, slot)
-
-	#the ground items are only displayable
-	if slot_item["Flags"] & IE_INV_ITEM_IDENTIFIED:
-		value = 0
-	else:
-		value = 2
-	DisplayItem (slot_item["ItemResRef"], value)
+	InventoryCommon.CloseItemInfoWindow ()
+	InventoryCommon.OpenItemInfoWindow ()
 	return
 
 def MouseEnterSlot ():
@@ -768,21 +513,5 @@ def MouseLeaveSlot ():
 			UpdateSlot (pc, i)
 	return
 
-def MouseEnterGround ():
-	Window = InventoryWindow
-	i = GemRB.GetVar ("GroundItemButton")
-	Button = Window.GetControl (i+47)
-	if GemRB.IsDraggingItem ():
-		Button.SetState (IE_GUI_BUTTON_SELECTED)
-	return
-
-def MouseLeaveGround ():
-	Window = InventoryWindow
-	i = GemRB.GetVar ("GroundItemButton")
-	Button = Window.GetControl (i+47)
-	if GemRB.IsDraggingItem ():
-		Button.SetState (IE_GUI_BUTTON_SECOND)
-	return
-
 ###################################################
 # End of file GUIINV.py
