@@ -28,12 +28,31 @@
 
 
 //this supposed to convince SDL to work on OS/X
+//WARNING: commenting this out will cause SDL 1.2.x to crash
 #ifdef __APPLE_CC__ // we need startup SDL here
-#include "SDL/SDL.h"
+#include <SDL.h>
+#endif
+
+#ifdef ANDROID
+#include <SDL/SDL.h>
+#include "audio.h"
+
+// pause audio playing if app goes in background
+static void appPutToBackground()
+{
+  core->GetAudioDrv()->Pause();
+}
+// resume audio playing if app return to foreground
+static void appPutToForeground()
+{
+  core->GetAudioDrv()->Resume();
+}
+
 #endif
 
 int main(int argc, char* argv[])
 {
+	Interface::SanityCheck(VERSION_GEMRB);
 	core = new Interface( argc, argv );
 	if (core->Init() == GEM_ERROR) {
 		delete( core );
@@ -42,6 +61,9 @@ int main(int argc, char* argv[])
 		getc(stdin);
 		return -1;
 	}
+#ifdef ANDROID
+    SDL_ANDROID_SetApplicationPutToBackgroundCallback(&appPutToBackground, &appPutToForeground);
+#endif
 	core->Main();
 	delete( core );
 	textcolor(DEFAULT);
