@@ -20,12 +20,14 @@
 
 #include "EffectQueue.h"
 
-#include "Actor.h"
+#include "DisplayMessage.h"
 #include "Effect.h"
 #include "Game.h"
 #include "Interface.h"
 #include "Map.h"
 #include "SymbolMgr.h"
+#include "Scriptable/Actor.h"
+#include "Spell.h"  //needs for the source flags bitfield
 
 #include <cstdio>
 
@@ -33,12 +35,13 @@ static struct {
 	const char* Name;
 	EffectFunction Function;
 	int Strref;
+	int Flags;
 } Opcodes[MAX_EFFECTS];
 
 static int initialized = 0;
-
-static EffectRef *effectnames = NULL;
+static EffectDesc *effectnames = NULL;
 static int effectnames_count = 0;
+static int pstflags = false;
 
 bool EffectQueue::match_ids(Actor *target, int table, ieDword value)
 {
@@ -52,6 +55,13 @@ bool EffectQueue::match_ids(Actor *target, int table, ieDword value)
 	case 2: //EA
 		stat = IE_EA; break;
 	case 3: //GENERAL
+		//this is a hack to support dead only projectiles in PST
+		//if it interferes with something feel free to remove
+		if (value==GEN_DEAD) {
+			if (target->GetStat(IE_STATE_ID)&STATE_DEAD) {
+				return true;
+			}
+		}
 		stat = IE_GENERAL; break;
 	case 4: //RACE
 		stat = IE_RACE; break;
@@ -163,48 +173,25 @@ int find_effect(const void *a, const void *b)
 	return stricmp((const char *) a,((const EffectRef *) b)->Name);
 }
 
-static EffectRef* FindEffect(const char* effectname)
+static EffectDesc* FindEffect(const char* effectname)
 {
 	if( !effectname || !effectnames) {
 		return NULL;
 	}
-	void *tmp = bsearch(effectname, effectnames, effectnames_count, sizeof(EffectRef), find_effect);
+	void *tmp = bsearch(effectname, effectnames, effectnames_count, sizeof(EffectDesc), find_effect);
 	if( !tmp) {
 		printMessage( "EffectQueue", "", YELLOW);
 		printf("Couldn't assign effect: %s\n", effectname );
 	}
-	return (EffectRef *) tmp;
+	return (EffectDesc *) tmp;
 }
 
-static EffectRef fx_protection_from_display_string_ref={"Protection:String",NULL,-1};
-
-//special effects without level check (but with damage dices precalculated)
-static EffectRef diced_effects[] = {
-	//core effects
-	{"Damage",NULL,-1},
-	{"CurrentHPModifier",NULL,-1},
-	{"MaximumHPModifier",NULL,-1},
-	//iwd effects
-	{"BurningBlood",NULL,-1}, //iwd
-	{"ColdDamage",NULL,-1},
-	{"CrushingDamage",NULL,-1},
-	{"VampiricTouch",NULL,-1},
-	{"VitriolicSphere",NULL,-1},
-	//pst effects
-	{"TransferHP",NULL,-1},
-{NULL,NULL,0} };
-
-//special effects without level check (but with damage dices not precalculated)
-static EffectRef diced_effects2[] = {
-	{"BurningBlood2",NULL,-1}, //how/iwd2
-	{"StaticCharge",NULL,-1}, //how/iwd2
-	{"LichTouch",NULL,-1}, //how
-{NULL,NULL,0} };
+static EffectRef fx_protection_from_display_string_ref = { "Protection:String", -1 };
 
 inline static void ResolveEffectRef(EffectRef &effect_reference)
 {
 	if( effect_reference.opcode==-1) {
-		EffectRef* ref = FindEffect(effect_reference.Name);
+		EffectDesc* ref = FindEffect(effect_reference.Name);
 		if( ref && ref->opcode>=0) {
 			effect_reference.opcode = ref->opcode;
 			return;
@@ -220,6 +207,8 @@ bool Init_EffectQueue()
 	if( initialized) {
 		return true;
 	}
+	pstflags = !!core->HasFeature(GF_PST_STATE_FLAGS);
+
 	memset( Opcodes, 0, sizeof( Opcodes ) );
 	for(i=0;i<MAX_EFFECTS;i++) {
 		Opcodes[i].Strref=-1;
@@ -253,10 +242,11 @@ bool Init_EffectQueue()
 			}
 		}
 
-		EffectRef* poi = FindEffect( effectname );
+		EffectDesc* poi = FindEffect( effectname );
 		if( poi != NULL) {
 			Opcodes[i].Function = poi->Function;
 			Opcodes[i].Name = poi->Name;
+			Opcodes[i].Flags = poi->Flags;
 			//reverse linking opcode number
 			//using this unused field
 			if( (poi->opcode!=-1) && effectname[0]!='*') {
@@ -269,14 +259,6 @@ bool Init_EffectQueue()
 	}
 	core->DelSymbol( eT );
 
-	//additional initialisations
-	for (i=0;diced_effects[i].Name;i++) {
-		ResolveEffectRef(diced_effects[i]);
-	}
-	for (i=0;diced_effects2[i].Name;i++) {
-		ResolveEffectRef(diced_effects2[i]);
-	}
-
 	return true;
 }
 
@@ -289,21 +271,21 @@ void EffectQueue_ReleaseMemory()
 	effectnames = NULL;
 }
 
-void EffectQueue_RegisterOpcodes(int count, const EffectRef* opcodes)
+void EffectQueue_RegisterOpcodes(int count, const EffectDesc* opcodes)
 {
 	if( ! effectnames) {
-		effectnames = (EffectRef*) malloc( (count+1) * sizeof( EffectRef ) );
+		effectnames = (EffectDesc*) malloc( (count+1) * sizeof( EffectDesc ) );
 	} else {
-		effectnames = (EffectRef*) realloc( effectnames, (effectnames_count + count + 1) * sizeof( EffectRef ) );
+		effectnames = (EffectDesc*) realloc( effectnames, (effectnames_count + count + 1) * sizeof( EffectDesc ) );
 	}
 
-	memcpy( effectnames + effectnames_count, opcodes, count * sizeof( EffectRef ));
+	memcpy( effectnames + effectnames_count, opcodes, count * sizeof( EffectDesc ));
 	effectnames_count += count;
 	effectnames[effectnames_count].Name = NULL;
 	//if we merge two effect lists, then we need to sort their effect tables
 	//actually, we might always want to sort this list, so there is no
 	//need to do it manually (sorted table is needed if we use bsearch)
-	qsort(effectnames, effectnames_count, sizeof(EffectRef), compare_effects);
+	qsort(effectnames, effectnames_count, sizeof(EffectDesc), compare_effects);
 }
 
 EffectQueue::EffectQueue()
@@ -419,7 +401,7 @@ Effect *EffectQueue::CreateEffectCopy(Effect *oldfx, EffectRef &effect_reference
 	return CreateEffectCopy(oldfx, effect_reference.opcode, param1, param2);
 }
 
-static EffectRef fx_unsummon_creature_ref={"UnsummonCreature",NULL,-1};
+static EffectRef fx_unsummon_creature_ref = { "UnsummonCreature", -1 };
 
 Effect *EffectQueue::CreateUnsummonEffect(Effect *fx)
 {
@@ -461,7 +443,7 @@ bool EffectQueue::RemoveEffect(Effect* fx)
 	for (std::list< Effect* >::iterator f = effects.begin(); f != effects.end(); f++ ) {
 		Effect* fx2 = *f;
 
-		if(  (fx==fx2) || !memcmp( fx, fx2, invariant_size)) {
+		if( (fx==fx2) || !memcmp( fx, fx2, invariant_size)) {
 			delete fx2;
 			effects.erase( f );
 			return true;
@@ -507,7 +489,7 @@ int EffectQueue::AddEffect(Effect* fx, Scriptable* self, Actor* pretarget, const
 	switch (fx->Target) {
 	case FX_TARGET_ORIGINAL:
 		fx->SetPosition(self->Pos);
-		
+
 		flg = ApplyEffect( st, fx, 1 );
 		if( fx->TimingMode != FX_DURATION_JUST_EXPIRED) {
 			if( st) {
@@ -606,9 +588,9 @@ int EffectQueue::AddEffect(Effect* fx, Scriptable* self, Actor* pretarget, const
 	case FX_TARGET_PARTY:
 all_party:
 		game = core->GetGame();
-		i = game->GetPartySize(true);
+		i = game->GetPartySize(false);
 		while(i--) {
-			Actor* actor = game->GetPC( i, true );
+			Actor* actor = game->GetPC( i, false );
 			fx->SetPosition(actor->Pos);
 
 			flg = ApplyEffect( actor, fx, 1 );
@@ -669,7 +651,7 @@ int EffectQueue::AddAllEffects(Actor* target, const Point &destination) const
 {
 	int res = FX_NOT_APPLIED;
 	// pre-roll dice for fx needing them and stow them in the effect
-	ieDword random_value = core->Roll( 1, 100, 0 );
+	ieDword random_value = core->Roll( 1, 100, -1 );
 
 	if( target) {
 		target->RollSaves();
@@ -698,39 +680,17 @@ int EffectQueue::AddAllEffects(Actor* target, const Point &destination) const
 	return res;
 }
 
-//check if an effect has no level based resistance, but instead the dice sizes/count
-//adjusts Parameter1 (like a damage causing effect)
-inline static bool IsDicedEffect(int opcode)
-{
-	int i;
-
-	for(i=0;diced_effects[i].Name;i++) {
-		if( diced_effects[i].opcode==opcode) {
-			return true;
-		}
-	}
-	return false;
-}
-
-//there is no level based resistance, but Parameter1 cannot be precalculated
-//these effects use the Dice fields in a special way
-inline static bool IsDicedEffect2(int opcode)
-{
-	int i;
-
-	for(i=0;diced_effects2[i].Name;i++) {
-		if( diced_effects2[i].opcode==opcode) {
-			return true;
-		}
-	}
-	return false;
-}
-
 //resisted effect based on level
 inline bool check_level(Actor *target, Effect *fx)
 {
 	//skip non level based effects
-	if( IsDicedEffect((int) fx->Opcode)) {
+	//check if an effect has no level based resistance, but instead the dice sizes/count
+	//adjusts Parameter1 (like a damage causing effect)
+	if( Opcodes[fx->Opcode].Flags & EFFECT_DICED ) {
+		//add the caster level to the dice count
+		if (fx->IsVariable) {
+			fx->DiceThrown+=fx->CasterLevel;
+		}
 		fx->Parameter1 = DICE_ROLL((signed)fx->Parameter1);
 		//this is a hack for PST style diced effects
 		if( core->HasFeature(GF_SAVE_FOR_HALF) ) {
@@ -744,33 +704,36 @@ inline bool check_level(Actor *target, Effect *fx)
 		}
 		return false;
 	}
-	if( IsDicedEffect2((int) fx->Opcode)) {
+	//there is no level based resistance, but Parameter1 cannot be precalculated
+	//these effects use the Dice fields in a special way
+	if( Opcodes[fx->Opcode].Flags & EFFECT_NO_LEVEL_CHECK ) {
 		return false;
 	}
 
 	if( !target) {
 		return false;
 	}
+	if(fx->Target == FX_TARGET_SELF) {
+		return false;
+	}
 
 	ieDword level = (ieDword) target->GetXPLevel( true );
 	//return true if resisted
-	//level resistance is checked when DiceSides or DiceThrown
-	//are greater than 0 (sometimes they used -1 for our amusement)
-	//if level>than maximum affected or level<than minimum affected, then the
-	//effect is resisted
-	if( (fx->DiceSides > 0 || fx->DiceThrown > 0) && (level > fx->DiceSides || level < fx->DiceThrown)) {
-		return true;
+	if (fx->MinAffectedLevel > 0 || fx->MaxAffectedLevel > 0) {
+		if (level < fx->MinAffectedLevel || level > fx->MaxAffectedLevel) {
+			return true;
+		}
 	}
 	return false;
 }
 
-//roll for the effect probability, there is a high and a low treshold, the d100 
+//roll for the effect probability, there is a high and a low treshold, the d100
 //roll should hit in the middle
 inline bool check_probability(Effect* fx)
 {
 	//watch for this, probability1 is the high number
 	//probability2 is the low number
-	//random value is 1-100
+	//random value is 0-99
 	if( fx->random_value<=fx->Probability2 || fx->random_value>fx->Probability1) {
 		return false;
 	}
@@ -778,35 +741,36 @@ inline bool check_probability(Effect* fx)
 }
 
 //immunity effects
-static EffectRef fx_level_immunity_ref={"Protection:Spelllevel",NULL,-1};
-static EffectRef fx_opcode_immunity_ref={"Protection:Opcode",NULL,-1}; //bg2
-static EffectRef fx_opcode_immunity2_ref={"Protection:Opcode2",NULL,-1};//iwd
-static EffectRef fx_spell_immunity_ref={"Protection:Spell",NULL,-1}; //bg2
-static EffectRef fx_spell_immunity2_ref={"Protection:Spell2",NULL,-1};//iwd
-static EffectRef fx_school_immunity_ref={"Protection:School",NULL,-1};
-static EffectRef fx_secondary_type_immunity_ref={"Protection:SecondaryType",NULL,-1};
+static EffectRef fx_level_immunity_ref = { "Protection:Spelllevel", -1 };
+static EffectRef fx_opcode_immunity_ref = { "Protection:Opcode", -1 }; //bg2
+static EffectRef fx_opcode_immunity2_ref = { "Protection:Opcode2", -1 };//iwd
+static EffectRef fx_spell_immunity_ref = { "Protection:Spell", -1 }; //bg2
+static EffectRef fx_spell_immunity2_ref = { "Protection:Spell2", -1 };//iwd
+static EffectRef fx_store_spell_sequencer_ref = { "Sequencer:Store", -1 }; //bg2, works against sequencers
+static EffectRef fx_school_immunity_ref = { "Protection:School", -1 };
+static EffectRef fx_secondary_type_immunity_ref = { "Protection:SecondaryType", -1 };
 
 //decrementing immunity effects
-static EffectRef fx_level_immunity_dec_ref={"Protection:SpellLevelDec",NULL,-1};
-static EffectRef fx_spell_immunity_dec_ref={"Protection:SpellDec",NULL,-1};
-static EffectRef fx_school_immunity_dec_ref={"Protection:SchoolDec",NULL,-1};
-static EffectRef fx_secondary_type_immunity_dec_ref={"Protection:SecondaryTypeDec",NULL,-1};
+static EffectRef fx_level_immunity_dec_ref = { "Protection:SpellLevelDec", -1 };
+static EffectRef fx_spell_immunity_dec_ref = { "Protection:SpellDec", -1 };
+static EffectRef fx_school_immunity_dec_ref = { "Protection:SchoolDec", -1 };
+static EffectRef fx_secondary_type_immunity_dec_ref = { "Protection:SecondaryTypeDec", -1 };
 
 //bounce effects
-static EffectRef fx_level_bounce_ref={"Bounce:SpellLevel",NULL,-1};
-//static EffectRef fx_opcode_bounce_ref={"Bounce:Opcode",NULL,-1};
-static EffectRef fx_spell_bounce_ref={"Bounce:Spell",NULL,-1};
-static EffectRef fx_school_bounce_ref={"Bounce:School",NULL,-1};
-static EffectRef fx_secondary_type_bounce_ref={"Bounce:SecondaryType",NULL,-1};
+static EffectRef fx_level_bounce_ref = { "Bounce:SpellLevel", -1 };
+//static EffectRef fx_opcode_bounce_ref = { "Bounce:Opcode", -1 };
+static EffectRef fx_spell_bounce_ref = { "Bounce:Spell", -1 };
+static EffectRef fx_school_bounce_ref = { "Bounce:School", -1 };
+static EffectRef fx_secondary_type_bounce_ref = { "Bounce:SecondaryType", -1 };
 
 //decrementing bounce effects
-static EffectRef fx_level_bounce_dec_ref={"Bounce:SpellLevelDec",NULL,-1};
-static EffectRef fx_spell_bounce_dec_ref={"Bounce:SpellDec",NULL,-1};
-static EffectRef fx_school_bounce_dec_ref={"Bounce:SchoolDec",NULL,-1};
-static EffectRef fx_secondary_type_bounce_dec_ref={"Bounce:SecondaryTypeDec",NULL,-1};
+static EffectRef fx_level_bounce_dec_ref = { "Bounce:SpellLevelDec", -1 };
+static EffectRef fx_spell_bounce_dec_ref = { "Bounce:SpellDec", -1 };
+static EffectRef fx_school_bounce_dec_ref = { "Bounce:SchoolDec", -1 };
+static EffectRef fx_secondary_type_bounce_dec_ref = { "Bounce:SecondaryTypeDec", -1 };
 
 //spelltrap (multiple decrementing immunity)
-static EffectRef fx_spelltrap={"SpellTrap", NULL,-1};
+static EffectRef fx_spelltrap = { "SpellTrap", -1 };
 
 //this is for whole spell immunity/bounce
 inline static void DecreaseEffect(Effect *efx)
@@ -849,6 +813,9 @@ static int check_type(Actor* actor, Effect* fx)
 		if( actor->fxqueue.HasEffectWithResource(fx_spell_immunity2_ref, fx->Source) ) {
 			return 0;
 		}
+		if (actor->fxqueue.HasEffectWithResource(fx_store_spell_sequencer_ref, fx->Source) ) {
+			return 0;
+		}
 	}
 
 	//primary type immunity (school)
@@ -1003,9 +970,14 @@ static bool check_resistance(Actor* actor, Effect* fx)
 		return false;
 	}
 
+	if (pstflags && (actor->GetSafeStat(IE_STATE_ID) & (STATE_ANTIMAGIC) ) ) {
+		return false;
+	}
+
 	//don't resist self
+	bool selective_mr = core->HasFeature(GF_SELECTIVE_MAGIC_RES);
 	if (fx->Target==FX_TARGET_SELF) {
-		if (core->HasFeature(GF_SELECTIVE_MAGIC_RES) ) {
+		if (selective_mr) {
 			return false;
 		}
 	}
@@ -1013,8 +985,11 @@ static bool check_resistance(Actor* actor, Effect* fx)
 	//magic immunity
 	ieDword val = actor->GetStat(IE_RESISTMAGIC);
 	if( fx->random_value < val) {
-		printf ("effect resisted: %s\n", (char*) Opcodes[fx->Opcode].Name);
-		return true;
+		// when using biased magic resistance non-hostile spells aren't resisted
+		if ((selective_mr && (fx->SourceFlags&SF_HOSTILE)) || !selective_mr) {
+			printf ("effect resisted: %s\n", (char*) Opcodes[fx->Opcode].Name);
+			return true;
+		}
 	}
 
 	//saving throws
@@ -1048,7 +1023,7 @@ static bool check_resistance(Actor* actor, Effect* fx)
 // returns FX_ABORT if the whole spell this effect is in should be aborted
 // it will disable all future effects of same source (only on first apply)
 
-int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply) const
+int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieDword resistance) const
 {
 	//printf( "FX 0x%02x: %s(%d, %d)\n", fx->Opcode, effectnames[fx->Opcode].Name, fx->Parameter1, fx->Parameter2 );
 	if( fx->Opcode >= MAX_EFFECTS) {
@@ -1060,26 +1035,44 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply) con
 
 	fx->FirstApply=first_apply;
 	if( first_apply) {
+		if (Owner)
+			fx->CasterID = Owner->GetGlobalID();
 		if( (fx->PosX==0xffffffff) && (fx->PosY==0xffffffff)) {
 			fx->PosX = target->Pos.x;
 			fx->PosY = target->Pos.y;
 		}
-		//the effect didn't pass the probability check
-		if( !check_probability(fx) ) {
-			fx->TimingMode = FX_DURATION_JUST_EXPIRED;
-			return FX_NOT_APPLIED;
+
+		//gemrb specific, stat based chance
+		if ((fx->Probability2 == 100) && Owner && (Owner->Type==ST_ACTOR) ) {
+			fx->Probability2 = 0;
+			fx->Probability1 = ((Actor *) Owner)->GetSafeStat(fx->Probability1);
 		}
 
-		//the effect didn't pass the target level check
-		if( check_level(target, fx) ) {
-			fx->TimingMode = FX_DURATION_JUST_EXPIRED;
-			return FX_NOT_APPLIED;
+		if (resistance) {
+			//the effect didn't pass the probability check
+			if( !check_probability(fx) ) {
+				fx->TimingMode = FX_DURATION_JUST_EXPIRED;
+				return FX_NOT_APPLIED;
+			}
+
+			//the effect didn't pass the target level check
+			if( check_level(target, fx) ) {
+				fx->TimingMode = FX_DURATION_JUST_EXPIRED;
+				return FX_NOT_APPLIED;
+			}
+
+			//the effect didn't pass the resistance check
+			if( check_resistance(target, fx) ) {
+				fx->TimingMode = FX_DURATION_JUST_EXPIRED;
+				return FX_NOT_APPLIED;
+			}
 		}
 
-		//the effect didn't pass the resistance check
-		if( check_resistance(target, fx) ) {
-			fx->TimingMode = FX_DURATION_JUST_EXPIRED;
-			return FX_NOT_APPLIED;
+		//Same as in items and spells
+		if (fx->SourceFlags & SF_HOSTILE) {
+			if (target && (target != Owner) && Owner && (Owner->Type==ST_ACTOR) ) {
+				target->AttackedBy((Actor *) Owner);
+			}
 		}
 
 		if( NeedPrepare(fx->TimingMode) ) {
@@ -1117,18 +1110,23 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply) con
 		break;
 	//this shouldn't happen
 	default:
+		printf("Unknown delay type: %d (from %d)\n", DelayType(fx->TimingMode&0xff), fx->TimingMode);
 		abort();
 	}
 
 	EffectFunction fn = 0;
 	if( fx->Opcode<MAX_EFFECTS) {
 		fn = Opcodes[fx->Opcode].Function;
+		if (!(target || (Opcodes[fx->Opcode].Flags & EFFECT_NO_ACTOR))) {
+			printf("targetless opcode without EFFECT_NO_ACTOR: %d, skipping\n", fx->Opcode);
+			return FX_NOT_APPLIED;
+		}
 	}
 	int res = FX_ABORT;
 	if( fn) {
-		if(  target && first_apply ) {
+		if( target && first_apply ) {
 			if( !target->fxqueue.HasEffectWithParamPair(fx_protection_from_display_string_ref, fx->Parameter1, 0) ) {
-				core->DisplayStringName( Opcodes[fx->Opcode].Strref, 0xf0f0f0,
+				displaymsg->DisplayStringName( Opcodes[fx->Opcode].Strref, 0xf0f0f0,
 					target, IE_STR_SOUND);
 			}
 		}
@@ -1141,7 +1139,7 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply) con
 				//normal effect with duration
 				break;
 			case FX_NOT_APPLIED:
- 				//instant effect, pending removal
+				//instant effect, pending removal
 				//for example, a damage effect
 				fx->TimingMode = FX_DURATION_JUST_EXPIRED;
 				break;
@@ -1339,8 +1337,8 @@ void EffectQueue::RemoveExpiredEffects(ieDword futuretime) const
 	for ( f = effects.begin(); f != effects.end(); f++ ) {
 		//FIXME: how this method handles delayed effects???
 		//it should remove them as well, i think
-		if(  DelayType( ((*f)->TimingMode) )!=PERMANENT ) {
-			if(  (*f)->Duration<=GameTime) {
+		if( DelayType( ((*f)->TimingMode) )!=PERMANENT ) {
+			if( (*f)->Duration<=GameTime) {
 				(*f)->TimingMode = FX_DURATION_JUST_EXPIRED;
 			}
 		}
@@ -1376,14 +1374,12 @@ void EffectQueue::RemoveAllEffectsWithParam(EffectRef &effect_reference, ieDword
 //remove certain levels of effects, possibly matching school/secondary type
 //this method removes whole spells (tied together by their source)
 //FIXME: probably this isn't perfect
-void EffectQueue::RemoveLevelEffects(ieDword level, ieDword Flags, ieDword match) const
+void EffectQueue::RemoveLevelEffects(ieResRef &Removed, ieDword level, ieDword Flags, ieDword match) const
 {
-	ieResRef Removed;
-
 	Removed[0]=0;
 	std::list< Effect* >::const_iterator f;
 	for ( f = effects.begin(); f != effects.end(); f++ ) {
-		if(  (*f)->Power>level) {
+		if( (*f)->Power>level) {
 			continue;
 		}
 
@@ -1500,7 +1496,7 @@ int EffectQueue::SpecificDamageBonus(ieDword opcode, ieDword param2) const
 	return bonus;
 }
 
-static EffectRef fx_damage_bonus_modifier_ref={"DamageBonusModifier",NULL,-1};
+static EffectRef fx_damage_bonus_modifier_ref = { "DamageBonusModifier", -1 };
 int EffectQueue::SpecificDamageBonus(ieDword damage_type) const
 {
 	ResolveEffectRef(fx_damage_bonus_modifier_ref);
@@ -1544,12 +1540,12 @@ int EffectQueue::BonusAgainstCreature(ieDword opcode, Actor *actor) const
 	for ( f = effects.begin(); f != effects.end(); f++ ) {
 		MATCH_OPCODE();
 		MATCH_LIVE_FX();
-		ieDword ids = (*f)->Parameter2;
-		if( ids<2 || ids>9) {
-			ids=2;
-		}
-		ieDword param1 = actor->GetStat(ids_stats[ids-2]);
-		if(  (*f)->Parameter1) {
+		if( (*f)->Parameter1) {
+			ieDword ids = (*f)->Parameter2;
+			if( ids<2 || ids>8) {
+				ids=2;
+			}
+			ieDword param1 = actor->GetStat(ids_stats[ids-2]);
 			MATCH_PARAM1();
 		}
 		int val = (int) (*f)->Parameter3;
@@ -1592,7 +1588,7 @@ bool EffectQueue::WeaponImmunity(ieDword opcode, int enchantment, ieDword weapon
 	return false;
 }
 
-static EffectRef fx_weapon_immunity_ref={"Protection:Weapons",NULL,-1};
+static EffectRef fx_weapon_immunity_ref = { "Protection:Weapons", -1 };
 
 bool EffectQueue::WeaponImmunity(int enchantment, ieDword weapontype) const
 {
@@ -1603,7 +1599,30 @@ bool EffectQueue::WeaponImmunity(int enchantment, ieDword weapontype) const
 	return WeaponImmunity(fx_weapon_immunity_ref.opcode, enchantment, weapontype);
 }
 
-static EffectRef fx_disable_spellcasting_ref={ "DisableCasting", NULL, -1 };
+void EffectQueue::AddWeaponEffects(EffectQueue *fxqueue, EffectRef &fx_ref) const
+{
+	ResolveEffectRef(fx_ref);
+	if( fx_ref.opcode<0) {
+		return;
+	}
+
+	ieDword opcode = fx_ref.opcode;
+	Point p(-1,-1);
+
+	std::list< Effect* >::const_iterator f;
+	for ( f = effects.begin(); f != effects.end(); f++ ) {
+		MATCH_OPCODE();
+		MATCH_LIVE_FX();
+		//
+		Effect *fx = core->GetEffect( (*f)->Resource, (*f)->Power, p);
+		if (!fx) continue;
+		fx->Target = FX_TARGET_PRESET;
+		fxqueue->AddEffect(fx, true);
+	}
+}
+
+/* no longer needed, use IE_CASTING stat
+static EffectRef fx_disable_spellcasting_ref = { "DisableCasting", -1 };
 int EffectQueue::DisabledSpellcasting(int types) const
 {
 	ResolveEffectRef(fx_disable_spellcasting_ref);
@@ -1653,6 +1672,7 @@ int EffectQueue::DisabledSpellcasting(int types) const
 	}
 	return spelltype_mask & types;
 }
+*/
 
 //useful for immunity vs spell, can't use item, etc.
 Effect *EffectQueue::HasOpcodeWithResource(ieDword opcode, const ieResRef resource) const
@@ -1743,7 +1763,7 @@ bool EffectQueue::HasDuration(Effect *fx)
 	return false;
 }
 
-static EffectRef fx_variable_ref={"Variable:StoreLocalVariable",NULL,-1};
+static EffectRef fx_variable_ref = { "Variable:StoreLocalVariable", -1 };
 
 //returns true if the effect must be saved
 //variables are saved differently
