@@ -33,18 +33,14 @@ class Game;
 #include "exports.h"
 #include "ie_types.h"
 
-#include "Actor.h"
+#include "Callback.h"
 #include "Map.h"
-#include "Particles.h"
 #include "Variables.h"
+#include "Scriptable/Actor.h"
 
 #include <vector>
 
-//the length of a round in game ticks?
-//default (ROUND_SIZE) is 6 seconds: 15 (AI_UPDATE_TIME)*6 (ROUND_SECODS)=90
-#define ROUND_SIZE     90
-#define ROUND_SECONDS  6
-#define ROUND_PER_TURN 10
+class Particles;
 
 //the size of the bestiary register
 #define BESTIARY_SIZE 260
@@ -56,6 +52,7 @@ class Game;
 //joinparty flags
 #define JP_JOIN     1  //refresh join time
 #define JP_INITPOS  2  //init startpos
+#define JP_SELECT   4  //select the actor after joining
 
 //protagonist mode
 #define PM_NO       0  //no death checks
@@ -98,6 +95,12 @@ class Game;
 #define REST_NOSCATTER  2 //no scatter check
 #define REST_NOMOVE     4 //no movement check
 #define REST_NOCRITTER  8 //no hostiles check
+
+//Song types (hardcoded)
+#define SONG_DAY        0
+#define SONG_NIGHT      1
+#define SONG_BATTLE     3
+
 /**
  * @struct PCStruct
  * Information about party member.
@@ -117,7 +120,7 @@ struct PCStruct {
 	ieWord   ViewYPos;
 	ieWord   ModalState;
 	ieWord   Happiness;
-	unsigned char Unknown2c[96];
+	ieDword  Interact[MAX_INTERACT];
 	ieWord   QuickWeaponSlot[MAX_QUICKWEAPONSLOT];
 	ieWord   QuickWeaponHeader[MAX_QUICKWEAPONSLOT];
 	ieResRef QuickSpellResRef[MAX_QSLOTS];
@@ -125,7 +128,7 @@ struct PCStruct {
 	ieWord   QuickItemHeader[MAX_QUICKITEMSLOT];
 	char Name[32];
 	ieDword  TalkCount;
-	ieByte QSlots[MAX_QSLOTS];
+	ieByte QSlots[GUIBT_COUNT];
 	ieByte QuickSpellClass[MAX_QSLOTS];
 };
 
@@ -154,6 +157,64 @@ struct GAMLocationEntry {
 	Point Pos;
 };
 
+//pst maze data structures (TODO: create a separate class?)
+struct maze_entry {
+	ieDword override;
+	ieDword accessible;
+	ieDword valid;
+	ieDword trapped;
+	ieDword traptype;
+	ieWord walls;
+	ieDword visited;
+};
+
+struct maze_header {
+	ieDword maze_sizex, maze_sizey;
+	ieDword pos1x, pos1y;  //nordom's position
+	ieDword pos2x, pos2y;  //main hall position
+	ieDword pos3x, pos3y;  //foyer entrance
+	ieDword pos4x, pos4y;  //unknown
+	ieDword trapcount;     //based on map size
+	ieDword initialized;   //set to 1
+	ieDword unknown2c;     //unknown
+	ieDword unknown30;     //unknown
+};
+
+#define MAZE_ENTRY_SIZE sizeof(maze_entry)
+#define MAZE_HEADER_SIZE sizeof(maze_header)
+#define MAZE_MAX_DIM 8
+#define MAZE_ENTRY_COUNT (MAZE_MAX_DIM*MAZE_MAX_DIM)
+#define MAZE_DATA_SIZE (MAZE_ENTRY_COUNT*MAZE_ENTRY_SIZE+MAZE_HEADER_SIZE)
+#define MAZE_DATA_SIZE_HARDCODED 1720
+
+//maze header indices
+#define MH_POS1X      0
+#define MH_POS1Y      1
+#define MH_POS2X      2
+#define MH_POS2Y      3
+#define MH_POS3X      4
+#define MH_POS3Y      5
+#define MH_POS4X      6
+#define MH_POS4Y      7
+#define MH_TRAPCOUNT  8
+#define MH_INITED     9
+#define MH_UNKNOWN2C  10
+#define MH_UNKNOWN30  11
+
+//maze entry indices
+#define ME_OVERRIDE   0
+#define ME_VALID      1
+#define ME_ACCESSIBLE 2
+#define ME_TRAP       3
+#define ME_WALLS      4
+#define ME_VISITED    5
+
+//ME_WALL bitfields
+#define WALL_SOUTH    1
+#define WALL_NORTH    2
+#define WALL_EAST     4
+#define WALL_WEST     8
+
 #define MAX_CRLEVEL 32
 
 typedef int CRRow[MAX_CRLEVEL];
@@ -184,7 +245,6 @@ private:
 public:
 	std::vector< Actor*> selected;
 	int version;
-	int Expansion;
 	Variables* kaputz;
 	ieByte* beasts;
 	ieByte* mazedata; //only in PST
@@ -211,17 +271,21 @@ public:
 	ieDword PartyGold;
 	ieWord NpcInParty;
 	ieWord WeatherBits;
-	ieDword Unknown48;
+	ieDword Unknown48; //still unknown
 	ieDword Reputation;
-	ieDword ControlStatus;// used in bg2, iwd (where you can switch panes off)
+	ieDword ControlStatus; // used in bg2, iwd (where you can switch panes off)
+	ieDword Expansion; // mostly used by BG2. IWD games set it to 3 on newgame
 	ieResRef AnotherArea;
 	ieResRef CurrentArea;
+	ieResRef PreviousArea; //move here if the worldmap exit is illegal?
 	ieResRef LoadMos;
 	Actor *timestop_owner;
 	ieDword timestop_end;
 	Particles *weather;
 	int event_timer;
-	char event_handler[64]; //like in Control
+	EventHandler event_handler; //like in Control
+	bool hasInfra;
+	bool familiarBlock;
 private:
 	/** reads the challenge rating table */
 	void LoadCRTable();
@@ -233,6 +297,8 @@ public:
 	/** Finds an actor in party by party ID, returns Actor, if not there, returns NULL*/
 	Actor* FindPC(unsigned int partyID);
 	Actor* FindNPC(unsigned int partyID);
+	/** Finds a global actor by global ID */
+	Actor* GetGlobalActorByGlobalID(ieDword globalID);
 	/** Finds an actor in party, returns slot, if not there, returns -1*/
 	int InParty(Actor* pc) const;
 	/** Finds an actor in store, returns slot, if not there, returns -1*/
@@ -282,7 +348,7 @@ public:
 	/** Returns slot of the map, if it was already loaded,
 	 * don't load it again, set changepf == true,
 	 * if you want to change the pathfinder too. */
-	int LoadMap(const char* ResRef);
+	int LoadMap(const char* ResRef, bool loadscreen);
 	int DelMap(unsigned int index, int forced = 0);
 	int AddNPC(Actor* npc);
 	Actor* GetNPC(unsigned int Index);
@@ -293,7 +359,9 @@ public:
 	int AttackersOf(ieDword globalID, Map *area) const;
 
 	//journal entries
+	/** Deletes one or all journal entries if strref is -1 */
 	void DeleteJournalEntry(ieStrRef strref);
+	/** Delete entries of the same group */
 	void DeleteJournalGroup(int Group);
 	/** Adds a journal entry from dialog data.
 	 * Time and chapter are calculated on the fly
@@ -305,10 +373,12 @@ public:
 	GAMJournalEntry* FindJournalEntry(ieStrRef strref);
 	GAMJournalEntry* GetJournalEntry(unsigned int Index);
 
+	//saved locations
 	unsigned int GetSavedLocationCount() const;
 	void ClearSavedLocations();
 	GAMLocationEntry* GetSavedLocationEntry(unsigned int Index);
 
+	//plane locations
 	unsigned int GetPlaneLocationCount() const;
 	void ClearPlaneLocations();
 	GAMLocationEntry* GetPlaneLocationEntry(unsigned int Index);
@@ -371,7 +441,7 @@ public:
 	/** Sets party size (1-32000) */
 	void SetPartySize(int value);
 	/** Sets a guiscript function to happen after x AI cycles have elapsed */
-	void SetTimedEvent(const char *fname, int count);
+	void SetTimedEvent(EventHandler func, int count);
 	/** Sets protagonist mode to 0-none,1-protagonist,2-team */
 	void SetProtagonistMode(int value);
 	void StartRainOrSnow(bool conditional, int weather);
@@ -387,19 +457,25 @@ public:
 	void RestParty(int checks, int dream, int hp);
 	/** timestop effect initiated by actor */
 	void TimeStop(Actor *actor, ieDword end);
+	/** updates the infravision info */
+	void Infravision();
 	/** gets the colour which should be applied over the game area,
 	may return NULL */
 	const Color *GetGlobalTint() const;
+	/** returns true if party has infravision */
+	bool PartyHasInfravision() const { return hasInfra; }
 	/** draw weather */
 	void DrawWeather(const Region &screen, bool update);
 	/** updates current area music */
-	void ChangeSong(bool force = true);
+	void ChangeSong(bool always = true, bool force = true);
 	/** sets expansion mode */
-	void SetExpansion(int exp);
+	void SetExpansion(ieDword value);
 	/** Dumps information about the object */
 	void DebugDump();
 	/** Finds an actor by global ID */
-	Actor *GetActorByGlobalID(ieWord objectID);
+	Actor *GetActorByGlobalID(ieDword objectID);
+	/** Allocates maze data */
+	ieByte *AllocateMazeData();
 private:
 	bool DetermineStartPosType(const TableMgr *strta);
 	ieResRef *GetDream(Map *area);
