@@ -25,12 +25,13 @@
 
 #include "win32def.h"
 
-#include "Actor.h"
-#include "GSUtils.h"
 #include "Game.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "Map.h"
+#include "GameScript/GSUtils.h"
+#include "GameScript/Matching.h"
+#include "Scriptable/Actor.h"
 
 static const int StatValues[9]={
 IE_EA, IE_FACTION, IE_TEAM, IE_GENERAL, IE_RACE, IE_CLASS, IE_SPECIFIC, 
@@ -447,6 +448,14 @@ void IniSpawn::ReadSpawnEntry(DataFileMgr *inifile, const char *entryname, Spawn
 	delete[] critters;
 }
 
+/* set by action */
+void IniSpawn::SetNamelessDeath(const ieResRef area, Point &pos, ieDword state)
+{
+	strnuprcpy(NamelessSpawnArea, area, 8);
+	NamelessSpawnPoint = pos;
+	NamelessState = state;
+}
+
 void IniSpawn::InitSpawn(const ieResRef DefaultArea)
 {
 	const char *s;
