@@ -33,6 +33,7 @@
 #include "exports.h"
 
 #include "Cache.h"
+#include "Callback.h"
 #include "GlobalTimer.h"
 #include "Holder.h"
 
@@ -43,6 +44,7 @@
 #include "Audio.h"
 #include "DataFileMgr.h"
 #include "MusicMgr.h"
+#include "SaveGame.h"
 #include "ScriptEngine.h"
 #include "StringMgr.h"
 #include "SymbolMgr.h"
@@ -58,7 +60,8 @@ class Container;
 class Control;
 class DataFileMgr;
 struct Effect;
-struct EffectRef;
+class EffectQueue;
+struct EffectDesc;
 class EventMgr;
 class Factory;
 class Font;
@@ -102,6 +105,7 @@ struct SlotType {
 	ieDword slottip;
 	ieDword slotid;
 	ieDword sloteffects;
+	ieDword slotflags;
 	ieResRef slotresref;
 };
 
@@ -122,6 +126,25 @@ struct ModalStatesStruct {
 	bool aoe_spell;
 };
 
+struct TimeStruct {
+	unsigned int round_sec;
+	unsigned int turn_sec;
+	unsigned int round_size; // in ticks
+	unsigned int rounds_per_turn;
+};
+
+struct SpellDescType {
+	ieResRef resref;
+	ieStrRef value;
+};
+#define SP_IDENTIFY  1      //any spell that cannot be cast from the menu
+#define SP_SILENCE   2      //any spell that can be cast in silence
+
+struct SurgeSpell {
+	ieResRef spell;
+	ieStrRef message;
+};
+
 class ItemList {
 public:
 	ieResRef *ResRefs;
@@ -176,7 +199,10 @@ public:
 #define EF_IDENTIFY      64       //starts identify screen
 #define EF_SELECTION     128      //selection changed
 #define EF_OPENSTORE     256      //open store window
-#define EF_MASTERSCRIPT  512      //change masterscript request
+#define EF_EXPANSION     512      //upgrade game request
+#define EF_CREATEMAZE    1024     //call the maze generator
+#define EF_RESETTARGET   2048     //reset the mouse cursor
+#define EF_TARGETMODE    4096     //update the mouse cursor
 
 //autopause
 #define AP_UNUSABLE      0
@@ -269,6 +295,7 @@ private:
 	std::vector<int> topwin;
 	Variables * vars;
 	Variables * tokens;
+	Variables * lists;
 	Holder<MusicMgr> music;
 	std::vector<Symbol> symbols;
 	Holder<DataFileMgr> INIparty;
@@ -278,13 +305,15 @@ private:
 	Game * game;
 	Calendar * calendar;
 	WorldMapArray* worldmap;
-	ieDword GameFeatures;   //the first 32 bits
-	ieDword GameFeatures2;  //the second 32 bits
+	ieDword GameFeatures[(GF_COUNT+31)/32];
+	//ieDword GameFeatures; //the first 32 bits
+	//ieDword GameFeatures2;//the second 32 bits
 	ieResRef ButtonFont;
 	ieResRef CursorBam;
 	ieResRef ScrollCursorBam;
 	ieResRef GroundCircleBam[MAX_CIRCLE_SIZE];
 	int GroundCircleScale[MAX_CIRCLE_SIZE];
+	ieDword FullScreen;
 	ieResRef TooltipFont;
 	ieResRef TooltipBackResRef;
 	ieResRef MovieFont;
@@ -316,6 +345,10 @@ private:
 	bool update_scripts;
 	/** Next Script Name */
 	char NextScript[64];
+	/** Function to call every main loop iteration */
+	EventHandler TickHook;
+	int SpecialSpellsCount;
+	SpellDescType *SpecialSpells;
 public:
 	Holder<StringMgr> strings;
 	GlobalTimer * timer;
@@ -323,11 +356,11 @@ public:
 	int SaveAsOriginal; //if true, saves files in compatible mode
 	int QuitFlag;
 	int EventFlag;
-	SaveGame *LoadGameIndex;
+	Holder<SaveGame> LoadGameIndex;
 	int VersionOverride;
 	unsigned int SlotTypes; //this is the same as the inventory size
 	ieResRef GlobalScript;
-	ieResRef WorldMapName;
+	ieResRef WorldMapName[2];
 	Variables * AreaAliasTable;
 	Variables * ItemExclTable;
 	Variables * ItemDialTable, *ItemDial2Table;
@@ -342,6 +375,8 @@ public:
 	std::vector<char *> musiclist;
 	std::multimap<ieDword, DamageInfoStruct> DamageInfoMap;
 	std::vector<ModalStatesStruct> ModalStates;
+	TimeStruct Time;
+	std::vector<SurgeSpell> SurgeSpells;
 public:
 	Interface(int iargc, char *iargv[]);
 	~Interface(void);
@@ -409,29 +444,6 @@ public:
 	Label *GetMessageLabel() const;
 	/** returns the textarea of the main game screen */
 	TextArea *GetMessageTextArea() const;
-	/** returns a string reference from a string reference index constant*/
-	ieStrRef GetStringReference(int stridx) const;
-	/** returns true if a string reference for a string reference index constant exists */
-	bool HasStringReference(int stridx) const;
-	/** returns the speaker's color and name */
-	unsigned int GetSpeakerColor(const char *&name, const Scriptable *&speaker) const;
-	/** displays any string in the textarea */
-	void DisplayString(const char *txt, Scriptable *speaker=NULL) const;
-	/** displays a string constant in the textarea */
-	void DisplayConstantString(int stridx, unsigned int color, Scriptable *speaker=NULL) const;
-	/** displays actor name - action : parameter */
-	void DisplayConstantStringNameString(int stridx, unsigned int color, int stridx2, const Scriptable *actor) const;
-	/** displays a string constant followed by a number in the textarea */
-	void DisplayConstantStringValue(int stridx, unsigned int color, ieDword value) const;
-	/** displays a string constant in the textarea, starting with speaker's name */
-	void DisplayConstantStringName(int stridx, unsigned int color, const Scriptable *speaker) const;
-	/** displays a string constant in the textarea, starting with actor, and ending with target */
-	void DisplayConstantStringAction(int stridx, unsigned int color, const Scriptable *actor, const Scriptable *target) const;
-	/** displays a string in the textarea */
-	void DisplayString(int stridx, unsigned int color, ieDword flags) const;
-	/** displays a string in the textarea, starting with speaker's name */
-	void DisplayStringName(int stridx, unsigned int color, const Scriptable *speaker, ieDword flags) const;
-	void DisplayStringName(const char *text, unsigned int color, const Scriptable *speaker) const;
 	/** returns the Window Visible Flag */
 	int GetVisible(unsigned short WindowIndex) const;
 	/** Set a Window Visible Flag */
@@ -523,13 +535,17 @@ public:
 	/** Gets the WorldMap class, returns the current worldmap or the first worldmap containing the area*/
 	WorldMap * GetWorldMap(const char *area = NULL);
 	void SetWindowFrame(int i, Sprite2D *Picture);
+	/** hides the game control window (if it exists), and reports success */
+	bool HideGCWindow();
+	/** unhides the game control window, if it exists */
+	void UnhideGCWindow();
 	GameControl *GetGameControl() const;
 	/** if backtomain is not null then goes back to main screen */
 	void QuitGame(int backtomain);
 	/** sets up load game */
-	void SetupLoadGame(SaveGame *save, int ver_override);
+	void SetupLoadGame(Holder<SaveGame> save, int ver_override);
 	/** load saved game by index (-1 is default), ver_override is an optional parameter
-	    to override the saved game's version */
+	to override the saved game's version */
 	void LoadGame(SaveGame *save, int ver_override);
 	/** fix changes in global script/worldmap*/
 	void UpdateMasterScript();
@@ -545,10 +561,11 @@ public:
 	ieDword QuerySlotType(unsigned int idx) const;
 	ieDword QuerySlottip(unsigned int idx) const;
 	ieDword QuerySlotID(unsigned int idx) const;
+	ieDword QuerySlotFlags(unsigned int idx) const;
 	ieDword QuerySlotEffects(unsigned int idx) const;
 	const char * QuerySlotResRef(unsigned int idx) const;
 	/*returns true if an itemtype is acceptable for a slottype, also checks the usability flags */
-	int CanUseItemType(int slottype, Item *item, Actor *actor=NULL, bool feedback=false) const;
+	int CanUseItemType(int slottype, Item *item, Actor *actor=NULL, bool feedback=false, bool equipped=false) const;
 	/*removes single file from cache*/
 	void RemoveFromCache(const ieResRef resref, SClass_ID SClassID);
 	/*removes all files from directory*/
@@ -571,6 +588,7 @@ public:
 	int GetDraggedPortrait() const { return DraggedPortrait; }
 	void SetDraggedPortrait(int dp, int cursor=-1);
 	CREItem *ReadItem(DataStream *str);
+	CREItem *ReadItem(DataStream *str, CREItem *itm);
 	bool ResolveRandomItem(CREItem *itm);
 	ieStrRef GetRumour(const ieResRef resname);
 	Container *GetCurrentContainer();
@@ -578,7 +596,7 @@ public:
 	void SetCurrentContainer(Actor *actor, Container *arg, bool flag=false);
 	Store *GetCurrentStore();
 	int CloseCurrentStore();
-	Store *SetCurrentStore(const ieResRef resname, const ieVariable owner);
+	Store *SetCurrentStore(const ieResRef resname, ieDword owner);
 	void SetMouseScrollSpeed(int speed);
 	int GetMouseScrollSpeed();
 	// FIXME: due to Win32 we have to allocate/release all common
@@ -597,6 +615,7 @@ public:
 	/** returns the first selected PC, if forced is set, then it returns
 	first PC if none was selected */
 	Actor *GetFirstSelectedPC(bool forced);
+	Actor *GetFirstSelectedActor();
 	/** returns a cursor sprite (not cached) */
 	Sprite2D *GetCursorSprite();
 	/** returns a scroll cursor sprite */
@@ -611,6 +630,8 @@ public:
 	int GetConstitutionBonus(int column, int value) const;
 	int GetCharismaBonus(int column, int value) const;
 	int GetLoreBonus(int column, int value) const;
+	int GetWisdomBonus(int column, int value) const;
+	int GetReputationMod(int column) const;
 
 	/** applies the spell on the target */
 	void ApplySpell(const ieResRef resname, Actor *target, Scriptable *caster, int level);
@@ -618,8 +639,9 @@ public:
 	void ApplySpellPoint(const ieResRef resname, Map *area, const Point &pos, Scriptable *caster, int level);
 	/** applies a single effect on the target */
 	int ApplyEffect(Effect *fx, Actor *target, Scriptable *caster);
-	//int ApplyEffect(const ieResRef resname, Actor *target, Scriptable *caster, int level, const Point &p);
-	//Actor *target, Scriptable *caster, 
+	/** applies an effect queue on the target */
+	int ApplyEffectQueue(EffectQueue *fxqueue, Actor *actor, Scriptable *caster);
+	int ApplyEffectQueue(EffectQueue *fxqueue, Actor *actor, Scriptable *caster, Point p);
 	Effect *GetEffect(const ieResRef resname, int level, const Point &p);
 	/** dumps an area object to the cache */
 	int SwapoutArea(Map *map);
@@ -634,7 +656,7 @@ public:
 	/** receives an autopause reason, returns 1 if pause was triggered by this call, -1 if it was already triggered */
 	int Autopause(ieDword reason);
 	/** registers engine opcodes */
-	void RegisterOpcodes(int count, const EffectRef *opcodes);
+	void RegisterOpcodes(int count, const EffectDesc *opcodes);
 	/** reads a list of resrefs into an array, returns array size */
 	int ReadResRefTable(const ieResRef tablename, ieResRef *&data);
 	/** frees the data */
@@ -665,6 +687,10 @@ public:
 	void StripLine(char * string, size_t size);
 	/** Returns the DeathVarFormat of the day */
 	static const char *GetDeathVarFormat();
+	int CheckSpecialSpell(ieResRef resref, Actor *actor);
+	int GetSpecialSpell(ieResRef resref);
+	int GetSpecialSpellsCount() { return SpecialSpellsCount; }
+	SpellDescType *GetSpecialSpells() { return SpecialSpells; }
 private:
 	int LoadSprites();
 	bool LoadConfig(void);
@@ -672,13 +698,15 @@ private:
 	bool LoadGemRBINI();
 	bool LoadINI(const char * filename);
 	bool InitItemTypes();
-	bool ReadStrrefs();
 	bool ReadRandomItems();
 	bool ReadItemTable(const ieResRef item, const char *Prefix);
 	bool ReadAbilityTables();
 	bool ReadAbilityTable(const ieResRef name, ieWordSigned *mem, int cols, int rows);
 	bool ReadMusicTable(const ieResRef name, int col);
 	bool ReadDamageTypeTable();
+	bool ReadReputationModTable();
+	bool ReadGameTimeTable();
+	bool ReadSpecialSpells();
 	bool ReadModalStates();
 	/** Reads table of area name mappings for WorldMap (PST only) */
 	bool ReadAreaAliasTable(const ieResRef name);
@@ -694,6 +722,8 @@ private:
 	GameControl* StartGameControl();
 	/** Executes everything (non graphical) in the main game loop */
 	void GameLoop(void);
+	/** the internal (without cache) part of GetListFrom2DA */
+	ieDword *GetListFrom2DAInternal(const ieResRef resref);
 public:
 	char GameDataPath[_MAX_PATH];
 	char GameOverridePath[_MAX_PATH];
@@ -714,15 +744,16 @@ public:
 	char PluginsPath[_MAX_PATH];
 	char CachePath[_MAX_PATH];
 	char GUIScriptsPath[_MAX_PATH];
-	char GamePath[_MAX_PATH];
 	char SavePath[_MAX_PATH];
 	char INIConfig[_MAX_PATH];
-	char CD[6][_MAX_PATH];
+	char GamePath[_MAX_PATH];
+	std::vector<std::string> CD[MAX_CD];
 	std::vector<std::string> ModPath;
 	int Width, Height, Bpp, ForceStereo;
 	unsigned int TooltipDelay;
+	int IgnoreOriginalINI;
 	unsigned int FogOfWar;
-	bool FullScreen, CaseSensitive, GameOnCD, SkipIntroVideos, DrawFPS;
+	bool CaseSensitive, GameOnCD, SkipIntroVideos, DrawFPS;
 	bool GUIEnhancements;
 	bool KeepCache;
 	Variables *plugin_flags;
@@ -749,6 +780,12 @@ public:
 	{
 		EventFlag|=Flag;
 	}
+	inline void ResetEventFlag(int Flag)
+	{
+		EventFlag&=~Flag;
+	}
+
+	static void SanityCheck(const char *ver);
 
 	/** Set Next Script */
 	void SetNextScript(const char *script);
@@ -761,6 +798,8 @@ public:
 
 	Audio* GetAudioDrv(void) const;
 
+	void SetTickHook(EventHandler);
+
 #ifdef _DEBUG
 	int FileStreamPtrCount;
 	int CachedFileStreamPtrCount;
