@@ -26,11 +26,11 @@ class Map;
 #include "exports.h"
 #include "globals.h"
 
-#include "ActorBlock.h"
 #include "Bitmap.h"
 #include "Image.h"
 #include "IniSpawn.h"
 #include "SpriteCover.h"
+#include "Scriptable/Scriptable.h"
 
 #include <queue>
 
@@ -151,6 +151,12 @@ public:
 	unsigned int GetCreatureCount() { return Count; }
 };
 
+class TerrainSounds {
+public:
+	ieResRef Group;
+	ieResRef Sounds[16];
+};
+
 class SpawnGroup {
 public:
 	ieResRef *ResRefs;
@@ -198,8 +204,9 @@ public:
 	void InitAnimation();
 	void SetPalette(ieResRef PaletteRef);
 	void BlendAnimation();
-	bool Schedule(ieDword gametime);
+	bool Schedule(ieDword gametime) const;
 	void Draw(const Region &screen, Map *area);
+	int GetHeight() const;
 private:
 	Animation *GetAnimationPiece(AnimationFactory *af, int animCycle);
 };
@@ -223,7 +230,6 @@ class GEM_EXPORT Map : public Scriptable {
 public:
 	TileMap* TMap;
 	Image* LightMap;
-	Bitmap* SearchMap;
 	Bitmap* HeightMap;
 	Sprite2D* SmallMap;
 	IniSpawn *INISpawn;
@@ -234,17 +240,20 @@ public:
 	ieByte* VisibleBitmap;
 	int version;
 	ieResRef WEDResRef;
-	ieWord localActorCounter;
 	bool MasterArea;
 	//this is set by the importer (not stored in the file)
 	bool DayNight;
 	//movies for day/night (only in ToB)
 	ieResRef Dream[2];
+	Sprite2D *Background;
+	ieDword BgDuration;
+
 private:
 	ieStrRef trackString;
 	int trackFlag;
 	ieWord trackDiff;
 	unsigned short* MapSet;
+	unsigned short* SrchMap; //internal searchmap
 	std::queue< unsigned int> InternalStack;
 	unsigned int Width, Height;
 	std::list< AreaAnimation*> animations;
@@ -277,17 +286,20 @@ public:
 	/* sets all the auxiliary maps and the tileset */
 	void AddTileMap(TileMap* tm, Image* lm, Bitmap* sr, Sprite2D* sm, Bitmap* hm);
 	void UpdateScripts();
+	void ResolveTerrainSound(ieResRef &sound, Point &pos);
 	bool DoStepForActor(Actor *actor, int speed, ieDword time);
 	void UpdateEffects();
 	/* removes empty heaps and returns total itemcount */
 	int ConsolidateContainers();
 	/* transfers all piles (loose items) to another map */
 	void CopyGroundPiles(Map *othermap, const Point &Pos);
+	/* transfers all ever visible piles (loose items) to the specified position */
+	void MoveVisibleGroundPiles(const Point &Pos);
 	/* draws stationary vvc graphics */
 	//void DrawVideocells(Region screen);
 	void DrawHighlightables(Region screen);
 	void DrawMap(Region screen);
-	void PlayAreaSong(int SongType, bool restart = true);
+	void PlayAreaSong(int SongType, bool restart = true, bool hard = false);
 	void AddAnimation(AreaAnimation* anim);
 	aniIterator GetFirstAnimation() { return animations.begin(); }
 	AreaAnimation* GetNextAnimation(aniIterator &iter)
@@ -316,8 +328,11 @@ public:
 	//returns true if an enemy is near P (used in resting/saving)
 	bool AnyEnemyNearPoint(const Point &p);
 	bool GetBlocked(unsigned int x, unsigned int y, unsigned int size);
-	unsigned char GetBlocked(unsigned int x, unsigned int y);
-	unsigned char GetBlocked(const Point &p);
+	unsigned int GetBlocked(unsigned int x, unsigned int y);
+	unsigned int GetBlocked(const Point &p);
+	Door *GetDoorByGlobalID(ieDword objectID);
+	Container *GetContainerByGlobalID(ieDword objectID);
+	InfoPoint *GetInfoPointByGlobalID(ieDword objectID);
 	Actor* GetActorByGlobalID(ieDword objectID);
 	Actor* GetActor(const Point &p, int flags);
 	Actor* GetActorInRadius(const Point &p, int flags, unsigned int radius);
@@ -326,6 +341,7 @@ public:
 	Actor* GetActor(int i, bool any);
 	Actor* GetActorByDialog(const char* resref);
 	Actor* GetActorByResource(const char* resref);
+	Actor* GetActorByScriptName(const char* name);
 	bool HasActor(Actor *actor);
 	void RemoveActor(Actor* actor);
 	//returns actors in rect (onlyparty could be more sophisticated)
@@ -333,6 +349,8 @@ public:
 	int GetActorCount(bool any) const;
 	//fix actors position if required
 	void JumpActors(bool jump);
+	//selects all selectable actors in the area
+	void SelectActors();
 	//if items == true, remove noncritical items from ground piles too
 	void PurgeArea(bool items);
 
@@ -348,8 +366,8 @@ public:
 	//get the next saved projectile
 	Projectile* GetNextTrap(proIterator &iter);
 	//add a projectile to the area
+	void AddProjectile(Projectile* pro, const Point &source, ieWord actorID, bool fake);
 	void AddProjectile(Projectile* pro, const Point &source, const Point &dest);
-	void AddProjectile(Projectile* pro, const Point &source, ieWord actorID);
 
 	//returns the duration of a VVC cell set in the area (point may be set to empty)
 	ieDword HasVVCCell(const ieResRef resource, const Point &p);
@@ -359,7 +377,7 @@ public:
 	//adds a sparkle puff of colour to a point in the area
 	//FragAnimID is an optional avatar animation ID (see avatars.2da) for
 	//fragment animation
-	void Sparkle(ieDword color, ieDword type, const Point &pos, unsigned int FragAnimID = 0);
+	void Sparkle(ieDword duration, ieDword color, ieDword type, const Point &pos, unsigned int FragAnimID = 0, int Zpos = 0);
 	//removes or fades the sparkle puff at a point
 	void FadeSparkle(const Point &pos, bool forced);
 
@@ -454,6 +472,9 @@ public:
 	//returns true if tracking failed
 	bool DisplayTrackString(Actor *actor);
 	unsigned int GetLightLevel(const Point &Pos);
+	unsigned short GetInternalSearchMap(int x, int y);
+	void SetInternalSearchMap(int x, int y, int value);
+	void SetBackground(const ieResRef &bgResref, ieDword duration);
 private:
 	AreaAnimation *GetNextAreaAnimation(aniIterator &iter, ieDword gametime);
 	Particles *GetNextSpark(spaIterator &iter);
