@@ -20,12 +20,12 @@
 
 #include "Spellbook.h"
 
-#include "Actor.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "Projectile.h"
 #include "Spell.h"
 #include "TableMgr.h"
+#include "Scriptable/Actor.h"
 
 #include <cstdio>
 
@@ -75,7 +75,7 @@ Spellbook::~Spellbook()
 			}
 		}
 	}
-	ClearSpellInfo(); 
+	ClearSpellInfo();
 	delete [] spells;
 }
 
@@ -92,6 +92,53 @@ void Spellbook::FreeSpellPage(CRESpellMemorization *sm)
 	delete sm;
 }
 
+// FIXME: exclude slayer, all bhaal innates?
+void Spellbook::CopyFrom(const Actor *source)
+{
+	if (!source) {
+		return;
+	}
+
+	// clear it first
+	for (int i = 0; i < NUM_BOOK_TYPES; i++) {
+		for (unsigned int j = 0; j < spells[i].size(); j++) {
+			if (spells[i][j]) {
+				FreeSpellPage( spells[i][j] );
+				spells[i][j] = NULL;
+			}
+		}
+		spells[i].clear();
+	}
+	ClearSpellInfo();
+
+	const Spellbook &wikipedia = source->spellbook;
+
+	for (int t = 0; t < NUM_BOOK_TYPES; t++) {
+		for (size_t i = 0; i < wikipedia.spells[t].size(); i++) {
+			unsigned int k;
+			CRESpellMemorization *wm = wikipedia.spells[t][i];
+			CRESpellMemorization *sm = new CRESpellMemorization();
+			spells[t].push_back(sm);
+			sm->Level = wm->Level;
+			sm->Number = wm->Number;
+			sm->Number2 = wm->Number2;
+			sm->Type = wm->Type;
+			for (k = 0; k < wm->known_spells.size(); k++) {
+				CREKnownSpell *tmp_known = new CREKnownSpell();
+				sm->known_spells.push_back(tmp_known);
+				memcpy(tmp_known, wm->known_spells[k], sizeof(CREKnownSpell));
+			}
+			for (k = 0; k < wm->memorized_spells.size(); k++) {
+				CREMemorizedSpell *tmp_mem = new CREMemorizedSpell();
+				sm->memorized_spells.push_back(tmp_mem);
+				memcpy(tmp_mem, wm->memorized_spells[k], sizeof(CREMemorizedSpell));
+			}
+		}
+	}
+
+	sorcerer = wikipedia.sorcerer;
+}
+
 //ITEM, SPPR, SPWI, SPIN, SPCL
 int sections[]={3,0,1,2,2};
 
@@ -207,6 +254,7 @@ bool Spellbook::IsIWDSpellBook() const
 
 unsigned int Spellbook::GetSpellLevelCount(int type) const
 {
+	assert(type < NUM_BOOK_TYPES);
 	return (unsigned int) spells[type].size();
 }
 
@@ -291,8 +339,8 @@ bool Spellbook::RemoveSpell(CREKnownSpell* spell)
 	return false;
 }
 
-
 //removes all instances of spellid (probably not needed)
+//IWD2 clab files use it
 void Spellbook::RemoveSpell(int spellid)
 {
 	int type = spellid/1000;
@@ -317,6 +365,7 @@ void Spellbook::RemoveSpell(int spellid)
 				(*sm)->known_spells.erase(ks);
 				RemoveMemorization(*sm, ResRef);
 				ks--;
+				ClearSpellInfo();
 			}
 		}
 	}
@@ -331,13 +380,14 @@ void Spellbook::RemoveSpell(const ieResRef ResRef)
 			std::vector< CREKnownSpell* >::iterator ks;
 
 			for (ks = (*sm)->known_spells.begin(); ks != (*sm)->known_spells.end(); ks++) {
-				if (strnicmp(ResRef, (*ks)->SpellResRef, sizeof(ResRef) ) ) {
+				if (strnicmp(ResRef, (*ks)->SpellResRef, sizeof(ieResRef) ) ) {
 					continue;
 				}
 				delete *ks;
 				(*sm)->known_spells.erase(ks);
 				RemoveMemorization(*sm, ResRef);
 				ks--;
+				ClearSpellInfo();
 			}
 		}
 	}
@@ -470,11 +520,9 @@ bool Spellbook::AddSpellMemorization(CRESpellMemorization* sm)
 	}
 
 	// only add this one if necessary
-	if (s->size() == level) {
-		s->push_back(sm);
-		return true;
-	}
-	return false;
+	assert (s->size() == level);
+	s->push_back(sm);
+	return true;
 }
 
 //apply the wisdom bonus on all spell levels for type
@@ -506,8 +554,11 @@ void Spellbook::ClearBonus()
 
 CRESpellMemorization *Spellbook::GetSpellMemorization(unsigned int type, unsigned int level)
 {
+	if (type >= (unsigned int)NUM_BOOK_TYPES)
+		return NULL;
+
 	CRESpellMemorization *sm;
-	if (level>= GetSpellLevelCount(type) || !(sm = spells[type][level]) ) {
+	if (level >= GetSpellLevelCount(type)) {
 		sm = new CRESpellMemorization();
 		sm->Type = (ieWord) type;
 		sm->Level = (ieWord) level;
@@ -516,6 +567,9 @@ CRESpellMemorization *Spellbook::GetSpellMemorization(unsigned int type, unsigne
 			delete sm;
 			return NULL;
 		}
+		assert(sm == spells[type][level]);
+	} else {
+		sm = spells[type][level];
 	}
 	return sm;
 }
@@ -593,6 +647,31 @@ bool Spellbook::UnmemorizeSpell(CREMemorizedSpell* spell)
 	return false;
 }
 
+bool Spellbook::UnmemorizeSpell(const ieResRef ResRef, bool deplete)
+{
+	for (int type = 0; type<NUM_BOOK_TYPES; type++) {
+		std::vector< CRESpellMemorization* >::iterator sm;
+		for (sm = spells[type].begin(); sm != spells[type].end(); sm++) {
+			std::vector< CREMemorizedSpell* >::iterator s;
+			for (s = (*sm)->memorized_spells.begin(); s != (*sm)->memorized_spells.end(); s++) {
+				if (strnicmp(ResRef, (*s)->SpellResRef, sizeof(ieResRef) ) ) {
+					continue;
+				}
+				if (deplete) {
+					(*s)->Flags = 0;
+				} else {
+					delete *s;
+					(*sm)->memorized_spells.erase( s );
+				}
+				ClearSpellInfo();
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
+
 //bitfield disabling type: 1 - mage, 2 - cleric etc
 //level: if set, then finds that level only
 CREMemorizedSpell* Spellbook::FindUnchargedSpell(int type, int level)
@@ -633,11 +712,11 @@ void Spellbook::CreateSorcererMemory(int type)
 			delete sm->memorized_spells[cnt];
 		}
 		sm->memorized_spells.clear();
-		for (unsigned int k = 0;  k < sm->known_spells.size(); k++) {
+		for (unsigned int k = 0; k < sm->known_spells.size(); k++) {
 			CREKnownSpell *ck = sm->known_spells[k];
 			cnt = sm->Number2;
 			while(cnt--) {
-			  MemorizeSpell(ck, true);
+				MemorizeSpell(ck, true);
 			}
 		}
 	}
@@ -812,6 +891,80 @@ SpellExtHeader *Spellbook::FindSpellInfo(unsigned int level, unsigned int type,
 	return NULL;
 }
 
+void Spellbook::AddSpellInfo(unsigned int sm_level, unsigned int sm_type, const ieResRef spellname, unsigned int idx)
+{
+	Spell *spl = gamedata->GetSpell(spellname);
+	if (!spl)
+		return;
+	if (spl->ExtHeaderCount<1)
+		return;
+
+	ieDword level = 0;
+	SpellExtHeader *seh = FindSpellInfo(sm_level, sm_type, spellname);
+	if (seh) {
+		seh->count++;
+		return;
+	}
+
+	seh = new SpellExtHeader;
+	spellinfo.push_back( seh );
+ 
+	memcpy(seh->spellname, spellname, sizeof(ieResRef) );
+	int ehc;
+	
+	for (ehc = 0; ehc < spl->ExtHeaderCount-1; ehc++) {
+		if (level<spl->ext_headers[ehc+1].RequiredLevel) {
+			break;
+		}
+	}
+	
+	SPLExtHeader *ext_header = spl->ext_headers+ehc;
+	seh->headerindex = ehc;
+	seh->level = sm_level;
+	seh->type = sm_type;
+	seh->slot = idx;
+	seh->count = 1;
+	seh->SpellForm = ext_header->SpellForm;
+	memcpy(seh->MemorisedIcon, ext_header->MemorisedIcon,sizeof(ieResRef) );
+	seh->Target = ext_header->Target;
+	seh->TargetNumber = ext_header->TargetNumber;
+	seh->Range = ext_header->Range;
+	seh->Projectile = ext_header->ProjectileAnimation;
+	seh->CastingTime = (ieWord) ext_header->CastingTime;
+	seh->strref = spl->SpellName;
+	gamedata->FreeSpell(spl, spellname, false);
+}
+
+void Spellbook::SetCustomSpellInfo(ieResRef *data, ieResRef spell, int type)
+{
+	ClearSpellInfo();
+	if (data) {
+		for(int i = 0; i<type;i++) {
+			AddSpellInfo(0,0,data[i],-1);
+		}
+		return;
+	}
+
+	//if data is not set, use the known spells list to set up the spellinfo list
+	for(int i = 0; i<NUM_BOOK_TYPES; i++) {
+		if ((1<<i)&type) {
+			for(unsigned int j = 0; j<spells[i].size(); j++) {
+				CRESpellMemorization* sm = spells[i][j];
+
+				for(unsigned int k=0;k<sm->known_spells.size(); k++) {
+					CREKnownSpell* slot = sm->known_spells[k];
+					if (!slot)
+						continue;
+					//skip the spell itself
+					if (!strnicmp(slot->SpellResRef, spell, sizeof(ieResRef)))
+						continue;
+					AddSpellInfo(sm->Level, sm->Type, slot->SpellResRef, -1);
+				}
+			}
+		}
+	}
+}
+
 // grouping the castable spells
 void Spellbook::GenerateSpellInfo()
 {
@@ -826,41 +979,7 @@ void Spellbook::GenerateSpellInfo()
 					continue;
 				if (!slot->Flags)
 					continue;
-				Spell *spl = gamedata->GetSpell(slot->SpellResRef);
-				if (!spl)
-					continue;
-				ieDword level = 0;
-				SpellExtHeader *seh = FindSpellInfo(sm->Level, sm->Type, slot->SpellResRef);
-				if (seh) {
-					seh->count++;
-					continue;
-				}
-				seh = new SpellExtHeader;
-				spellinfo.push_back( seh );
-
-				memcpy(seh->spellname, slot->SpellResRef, sizeof(ieResRef) );
-				int ehc;
-
-				for (ehc = 0; ehc < spl->ExtHeaderCount-1; ehc++) {
-					if (level<spl->ext_headers[ehc+1].RequiredLevel) {
-						break;
-					}
-				}
-				SPLExtHeader *ext_header = spl->ext_headers+ehc;
-				seh->headerindex = ehc;
-				seh->level = sm->Level;
-				seh->type = sm->Type;
-				seh->slot = k;
-				seh->count = 1;
-				seh->SpellForm = ext_header->SpellForm;
-				memcpy(seh->MemorisedIcon, ext_header->MemorisedIcon,sizeof(ieResRef) );
-				seh->Target = ext_header->Target;
-				seh->TargetNumber = ext_header->TargetNumber;
-				seh->Range = ext_header->Range;
-				seh->Projectile = ext_header->ProjectileAnimation;
-				seh->CastingTime = (ieWord) ext_header->CastingTime;
-				seh->strref = spl->SpellName;
-				gamedata->FreeSpell(spl, slot->SpellResRef, false);
+				AddSpellInfo(sm->Level, sm->Type, slot->SpellResRef, k);
 			}
 		}
 	}
@@ -874,12 +993,6 @@ void Spellbook::dump()
 	for (int i = 0; i < NUM_BOOK_TYPES; i++) {
 		for (unsigned int j = 0; j < spells[i].size(); j++) {
 			CRESpellMemorization* sm = spells[i][j];
-			//if (!sm || !sm->Number) continue;
-			if (!sm) continue;
-
-			//Never ever use field length qualifiers it is not portable, if you need to convert, convert to compatible values, anyway we don't need this!
-			//printf ( "type: %d: L: %d; N1: %d; N2: %d; T: %d; KC: %d; MC: %d\n", i,
-			//	 sm->Level, sm->Number, sm->Number2, sm->Type, (int) sm->known_spells.size(), (int) sm->memorized_spells.size() );
 
 			if (sm->known_spells.size())
 				printf( " Known spells:\n" );
