@@ -171,6 +171,33 @@ void WorldMap::SetAreaEntry(unsigned int x, WMPAreaEntry *ae)
 	area_entries.push_back(ae);
 }
 
+void WorldMap::InsertAreaLink(unsigned int areaidx, unsigned int dir, WMPAreaLink *arealink)
+{
+	unsigned int pos;
+	WMPAreaEntry *ae;
+
+	WMPAreaLink *al = new WMPAreaLink();
+	memcpy(al, arealink, sizeof(WMPAreaLink) );
+	unsigned int idx = area_entries[areaidx]->AreaLinksIndex[dir];
+	area_links.insert(area_links.begin()+idx,al);
+
+	unsigned int max = area_entries.size();
+	for(pos = 0; pos<max; pos++) {
+		ae = area_entries[pos];
+		for (unsigned int k=0;k<4;k++) {
+			if ((pos==areaidx) && (k==dir)) {
+				ae->AreaLinksCount[k]++;
+				continue;
+			}
+			if(ae->AreaLinksIndex[k]>=idx) {
+				ae->AreaLinksIndex[k]++;
+			}
+		}
+	}
+	//update the link count, just in case
+	AreaLinksCount++;
+}
+
 void WorldMap::SetAreaLink(unsigned int x, WMPAreaLink *arealink)
 {
 	WMPAreaLink *al =new WMPAreaLink();
@@ -239,11 +266,41 @@ WMPAreaEntry* WorldMap::GetArea(const ieResRef AreaName, unsigned int &i) const
 	return NULL;
 }
 
+//Find Worldmap location by nearest area with a smaller number
+//Counting backwards, stop at 1000 boundaries.
+//It is not possible to simply round to 1000, because there are 
+//WMP entries like AR8001, and we need to find the best match
+WMPAreaEntry* WorldMap::FindNearestEntry(const ieResRef AreaName, unsigned int &i) const
+{
+	int value = 0;
+	ieResRef tmp;
+
+	sscanf(&AreaName[2],"%4d", &value);
+	do {
+		snprintf(tmp, 9, "%.2s%04d", AreaName, value);
+		WMPAreaEntry* ret = GetArea(tmp, i);
+		if (ret) {
+			return ret;
+		}
+		if (value%1000 == 0) break;
+		value--;
+	}
+	while(1); //value%1000 should protect us from infinite loops
+	i = -1;
+	return NULL;
+}
+
 //this is a pathfinding algorithm
 //you have to find the optimal path
-
 int WorldMap::CalculateDistances(const ieResRef AreaName, int direction)
 {
+	//first, update reachable/visible areas by worlde.2da if exists
+	UpdateReachableAreas();
+	UpdateAreaVisibility(AreaName, direction);
+	if (direction==-1) {
+		return 0;
+	}
+
 	if (direction<0 || direction>3) {
 		printMessage("WorldMap","", LIGHT_RED);
 		printf("CalculateDistances for invalid direction: %s\n", AreaName);
@@ -265,8 +322,6 @@ int WorldMap::CalculateDistances(const ieResRef AreaName, int direction)
 
 	printMessage("WorldMap","", GREEN);
 	printf("CalculateDistances for Area: %s\n", AreaName);
-	UpdateReachableAreas();
-	UpdateAreaVisibility(AreaName, direction);
 
 	size_t memsize =sizeof(int) * area_entries.size();
 	Distances = (int *) malloc( memsize );
@@ -289,7 +344,7 @@ int WorldMap::CalculateDistances(const ieResRef AreaName, int direction)
 		for(int d=0;d<4;d++) {
 			int j=ae->AreaLinksIndex[d];
 			int k=j+ae->AreaLinksCount[d];
-			if ((size_t) k>=area_links.size()) {
+			if ((size_t) k>area_links.size()) {
 				printMessage("WorldMap","The worldmap file is corrupted... and it would crash right now!\n",RED);
 				printf("Entry #: %d Direction: %d\n",i,d);
 				break;
@@ -427,9 +482,8 @@ int WorldMap::GetDistance(const ieResRef AreaName) const
 
 void WorldMap::UpdateAreaVisibility(const ieResRef AreaName, int direction)
 {
-	if (direction<0 || direction>3)
-		return;
 	unsigned int i;
+
 	WMPAreaEntry* ae=GetArea(AreaName,i);
 	if (!ae)
 		return;
@@ -437,6 +491,8 @@ void WorldMap::UpdateAreaVisibility(const ieResRef AreaName, int direction)
 	printf("Updated Area visibility: %s (visited, and visible)\n", AreaName);
 
 	ae->SetAreaStatus(WMP_ENTRY_VISITED|WMP_ENTRY_VISIBLE, BM_OR);
+	if (direction<0 || direction>3)
+		return;
 	i=ae->AreaLinksCount[direction];
 	while (i--) {
 		WMPAreaLink* al = area_links[ae->AreaLinksIndex[direction]+i];
@@ -486,11 +542,14 @@ WorldMapArray::WorldMapArray(unsigned int count)
 	CurrentMap = 0;
 	MapCount = count;
 	all_maps = (WorldMap **) calloc(count, sizeof(WorldMap *) );
+	single = true;
 }
 
 WorldMapArray::~WorldMapArray()
 {
-	for (unsigned int i = 0; i<MapCount; i++) {
+	unsigned int i;
+
+	for (i = 0; i<MapCount; i++) {
 		if (all_maps[i]) {
 			delete all_maps[i];
 		}
