@@ -34,10 +34,17 @@
 
 #include "ie_types.h"
 
-#define VERSION_GEMRB "0.6.1"
+#define VERSION_GEMRB "0.6.4"
 
 #define GEMRB_STRING "GemRB v" VERSION_GEMRB
 
+#ifdef ANDROID
+# define PACKAGE "GemRB"
+# define S_IEXEC  S_IXUSR
+# define S_IREAD  S_IRUSR
+# define S_IWRITE S_IWUSR
+#endif
+
 #ifndef GLOBALS_ONLY_DEFS
 
 #include "RGBAColor.h"
@@ -46,11 +53,9 @@
 #include "win32def.h"
 
 #include "AnimStructures.h"
-#include "DataStream.h"
+#include "System/DataStream.h"
 #include "Region.h"
 #include "Sprite2D.h"
-#include "VideoMode.h"
-#include "VideoModes.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -119,7 +124,7 @@
 #define  GF_IWD2_DEATHVARFORMAT  	34 //iwd branch (maybe pst)
 #define  GF_RESDATA_INI         	35 //pst
 #define  GF_OVERRIDE_CURSORPOS  	36 //pst, iwd2
-#define  GF_BREAKABLE_WEAPONS     	37 //bg1, not bg2 and iwd2, maybe others
+#define  GF_BREAKABLE_WEAPONS     	37 //only bg1
 #define  GF_3ED_RULES              	38 //iwd2
 #define  GF_LEVELSLOT_PER_CLASS    	39 //iwd2
 #define  GF_SELECTIVE_MAGIC_RES    	40 //bg2, iwd2, (how)
@@ -129,8 +134,23 @@
 #define  GF_ONSCREEN_TEXT       	44 //pst
 #define  GF_SPECIFIC_DMG_BONUS		45 //how, iwd2
 #define  GF_STRREF_SAVEGAME       	46 //iwd2
-//update this
-#define GF_COUNT 47
+#define  GF_WISDOM_BONUS      	 	47 //pst
+#define  GF_BIOGRAPHY_RES               48 //iwd branch
+#define  GF_NO_BIOGRAPHY                49 //pst
+#define  GF_STEAL_IS_ATTACK             50 //bg2 for sure
+#define  GF_CUTSCENE_AREASCRIPTS	51 //bg1, maybe more
+#define  GF_FLEXIBLE_WMAP               52 //iwd
+#define  GF_AUTOSEARCH_HIDDEN           53 //all except iwd2
+#define  GF_PST_STATE_FLAGS             54 //pst complicates this
+#define  GF_NO_DROP_CAN_MOVE            55 //bg1
+#define  GF_JOURNAL_HAS_SECTIONS        56 //bg2
+#define  GF_CASTING_SOUNDS              57 //all except pst and bg1
+#define  GF_CASTING_SOUNDS2             58 //bg2
+#define  GF_FORCE_AREA_SCRIPT           59 //how and iwd2 (maybe iwd1)
+#define  GF_AREA_OVERRIDE               60 //pst maze and other hardcode
+#define  GF_NO_NEW_VARIABLES            61 //pst
+//update this or bad things can happen
+#define GF_COUNT 62
 
 //the number of item usage fields (used in CREItem and STOItem)
 #define CHARGE_COUNTERS  3
@@ -160,9 +180,14 @@ GEM_EXPORT int strnlen(const char* string, int maxlen);
 GEM_EXPORT unsigned char GetOrient(const Point &s, const Point &d);
 GEM_EXPORT unsigned int Distance(const Point pos, const Point pos2);
 GEM_EXPORT unsigned int Distance(const Point pos, Scriptable *b);
+GEM_EXPORT unsigned int SquaredMapDistance(const Point pos, Scriptable *b);
 GEM_EXPORT unsigned int PersonalDistance(const Point pos, Scriptable *b);
+GEM_EXPORT unsigned int SquaredPersonalDistance(const Point pos, Scriptable *b);
 GEM_EXPORT unsigned int Distance(Scriptable *a, Scriptable *b);
+GEM_EXPORT unsigned int SquaredDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT unsigned int PersonalDistance(Scriptable *a, Scriptable *b);
+GEM_EXPORT unsigned int SquaredPersonalDistance(Scriptable *a, Scriptable *b);
+GEM_EXPORT unsigned int SquaredMapDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT int EARelation(Scriptable *a, Actor *b);
 GEM_EXPORT bool dir_exists(const char* path);
 GEM_EXPORT int strlench(const char* string, char ch);
@@ -187,8 +212,6 @@ GEM_EXPORT char* strlwr(char* string);
 	}
 #endif
 
-//struct ActorBlock;
-
 inline int MIN(int a, int b)
 {
 	return (a > b ? b : a);
@@ -201,15 +224,18 @@ inline int MAX(int a, int b)
 
 inline bool valid_number(const char* string, long& val)
 {
-        char* endpr;
+	char* endpr;
 
-        val = (long) strtoul( string, &endpr, 0 );
-        return ( const char * ) endpr != string;
+	val = (long) strtoul( string, &endpr, 0 );
+	return ( const char * ) endpr != string;
 }
 
 //we need 32+6 bytes at least, because we store 'context' in the variable
 //name too
 #define MAX_VARIABLE_LENGTH  40
 
+//the maximum supported game CD count
+#define MAX_CD               6
+
 #endif //! GLOBALS_H
 
