@@ -24,12 +24,12 @@
 
 #include "AnimationFactory.h"
 #include "Audio.h"
-#include "Console.h"
 #include "Game.h" // for GetGlobalTint
 #include "GameData.h"
 #include "Interface.h"
 #include "Palette.h"
 #include "SpriteCover.h"
+#include "GUI/Console.h"
 
 #include <cmath>
 #include <cassert>
@@ -94,7 +94,7 @@ int SDLVideoDriver::CreateDisplay(int w, int h, int b, bool fs)
 	bpp=b;
 	fullscreen=fs;
 	printMessage( "SDLVideo", "Creating display\n", WHITE );
-	ieDword flags = SDL_HWSURFACE | SDL_DOUBLEBUF;
+	ieDword flags = SDL_SWSURFACE;
 	if (fullscreen) {
 		flags |= SDL_FULLSCREEN;
 	}
@@ -117,7 +117,7 @@ int SDLVideoDriver::CreateDisplay(int w, int h, int b, bool fs)
 	Viewport.w = width;
 	Viewport.h = height;
 	printMessage( "SDLVideo", "Creating Main Surface...", WHITE );
-	SDL_Surface* tmp = SDL_CreateRGBSurface( SDL_HWSURFACE, width, height,
+	SDL_Surface* tmp = SDL_CreateRGBSurface( SDL_SWSURFACE, width, height,
 						bpp, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff );
 	printStatus( "OK", LIGHT_GREEN );
 	printMessage( "SDLVideo", "Creating Back Buffer...", WHITE );
@@ -141,74 +141,19 @@ void SDLVideoDriver::SetDisplayTitle(char* title, char* icon)
 	SDL_WM_SetCaption( title, icon );
 }
 
-VideoModes SDLVideoDriver::GetVideoModes(bool fullscreen)
-{
-	SDL_Rect** modes;
-	ieDword flags = SDL_HWSURFACE;
-	if (fullscreen) {
-		flags |= SDL_FULLSCREEN;
-	}
-	VideoModes vm;
-	//32-bit Video Modes
-	SDL_PixelFormat pf;
-	pf.palette = NULL;
-	pf.BitsPerPixel = 32;
-	pf.BytesPerPixel = 4;
-	pf.Rmask = 0xff000000;
-	pf.Gmask = 0x00ff0000;
-	pf.Bmask = 0x0000ff00;
-	pf.Amask = 0x000000ff;
-	pf.Rshift = 24;
-	pf.Gshift = 16;
-	pf.Bshift = 8;
-	pf.Ashift = 0;
-	modes = SDL_ListModes( &pf, fullscreen );
-	if (modes == ( SDL_Rect * * ) 0) {
-		return vm;
-	}
-	if (modes == ( SDL_Rect * * ) - 1) {
-		vm.AddVideoMode( 640, 480, 32, fullscreen );
-		vm.AddVideoMode( 800, 600, 32, fullscreen );
-		vm.AddVideoMode( 1024, 786, 32, fullscreen );
-		vm.AddVideoMode( 1280, 1024, 32, fullscreen );
-		vm.AddVideoMode( 1600, 1200, 32, fullscreen );
-	} else {
-		for (int i = 0; modes[i]; i++) {
-			vm.AddVideoMode( modes[i]->w, modes[i]->h, 32, fullscreen );
-		}
-	}
-	return vm;
-}
-
-bool SDLVideoDriver::TestVideoMode(VideoMode& vm)
-{
-	ieDword flags = SDL_HWSURFACE;
-	if (vm.GetFullScreen()) {
-		flags |= SDL_FULLSCREEN;
-	}
-	if (SDL_VideoModeOK( vm.GetWidth(), vm.GetHeight(), vm.GetBPP(), flags ) == vm.GetBPP()) {
-		return true;
-	}
-	return false;
-}
-
 bool SDLVideoDriver::ToggleFullscreenMode(int set_reset)
 {
 	if (set_reset==-1) {
-		 set_reset=fullscreen;
+		 set_reset=!fullscreen;
 	}
-	if (fullscreen == !set_reset) {
+	if (fullscreen != set_reset) {
 		fullscreen=set_reset;
-		ieDword flags = SDL_HWSURFACE | SDL_DOUBLEBUF;
-		if (fullscreen) {
-			flags |= SDL_FULLSCREEN;
-		}
-		SDL_Surface* disp2 = SDL_SetVideoMode( width, height, bpp, flags );
-		if (disp2) {
-			disp = disp2;
-			MoveMouse(CursorPos.x,CursorPos.y);
-			return true;
-		}
+		SDL_WM_ToggleFullScreen( disp );
+		//readjust mouse to original position
+		MoveMouse(CursorPos.x, CursorPos.y);
+		//synchronise internal variable
+		core->GetDictionary()->SetAt( "Full Screen", (ieDword) fullscreen );
+		return true;
 	}
 	return false;
 }
@@ -224,10 +169,6 @@ inline int GetModState(int modstate)
 
 int SDLVideoDriver::SwapBuffers(void)
 {
-	static bool lastevent = false; /* last event was a mousedown */
-	static unsigned long lastmousetime = 0;
-	int x,y;
-
 	int ret = GEM_OK;
 	unsigned long time;
 	GetTime( time );
@@ -237,14 +178,75 @@ int SDLVideoDriver::SwapBuffers(void)
 	}
 	lastTime = time;
 
-	unsigned char key = 0;
 	bool ConsolePopped = core->ConsolePopped;
 
 	if (ConsolePopped) {
 		core->DrawConsole();
 	}
 
+	ret = PollEvents();
+
+	SDL_BlitSurface( backBuf, NULL, disp, NULL );
+	if (fadeColor.a) {
+		SDL_SetAlpha( extra, SDL_SRCALPHA, fadeColor.a );
+		SDL_Rect src = {
+			0, 0, Viewport.w, Viewport.h
+		};
+		SDL_Rect dst = {
+			xCorr, yCorr, 0, 0
+		};
+		SDL_BlitSurface( extra, &src, disp, &dst );
+	}
+
+	if (Cursor[CursorIndex] && !(DisableMouse&MOUSE_DISABLED)) {
+		SDL_Surface* temp = backBuf;
+		backBuf = disp; // FIXME: UGLY HACK!
+		if (DisableMouse&MOUSE_GRAYED) {
+			//used for greyscale blitting, fadeColor is unused
+			BlitGameSprite(Cursor[CursorIndex], CursorPos.x, CursorPos.y, BLIT_GREY, fadeColor, NULL, NULL, NULL, true);
+		} else {
+			BlitSprite(Cursor[CursorIndex], CursorPos.x, CursorPos.y, true);
+		}
+		backBuf = temp;
+	}
+
+	//handle tooltips
+	unsigned int delay = core->TooltipDelay;
+	// The multiplication by 10 is there since the last, disabling slider position is the eleventh
+	if (!ConsolePopped && (delay<TOOLTIP_DELAY_FACTOR*10) ) {
+		GetTime( time );
+		/** Display tooltip if mouse is idle */
+		if (( time - lastMouseTime ) > delay) {
+			if (Evnt)
+				Evnt->MouseIdle( time - lastMouseTime );
+		}
+
+		/** This causes the tooltip to be rendered directly to display */
+		SDL_Surface* tmp = backBuf;
+		backBuf = disp; // FIXME: UGLY HACK!
+		core->DrawTooltip();
+		backBuf = tmp;
+	}
+
+	SDL_Flip( disp );
+
+	return ret;
+}
+
+int SDLVideoDriver::PollEvents() {
+	static bool lastevent = false; /* last event was a mousedown */
+	static unsigned long lastmousetime = 0;
+
+	int ret = GEM_OK;
+
+	bool ConsolePopped = core->ConsolePopped;
+	unsigned long time = lastTime;
+	unsigned char key = 0;
+	int x,y;
+
 	while ( SDL_PollEvent(&event) ) {
+		int modstate = GetModState(event.key.keysym.mod);
+
 		/* Loop until there are no events left on the queue */
 		switch (event.type) {
 		/* Process the appropriate event type */
@@ -255,13 +257,18 @@ int SDLVideoDriver::SwapBuffers(void)
 
 		case SDL_KEYUP:
 			switch(event.key.keysym.sym) {
-				case SDLK_LSHIFT:
-				case SDLK_RSHIFT:
+				case SDLK_LALT:
+				case SDLK_RALT:
 					key = GEM_ALT;
 					break;
 				case SDLK_SCROLLOCK:
 					key = GEM_GRAB;
 					break;
+				case SDLK_f:
+					if (modstate & GEM_MOD_CTRL) {
+						ToggleFullscreenMode(-1);
+					}
+					break;
 				default:
 					if (event.key.keysym.sym<256) {
 						key=(unsigned char) event.key.keysym.sym;
@@ -269,11 +276,11 @@ int SDLVideoDriver::SwapBuffers(void)
 					break;
 			}
 			if (!ConsolePopped && Evnt && ( key != 0 ))
-				Evnt->KeyRelease( key, GetModState(event.key.keysym.mod) );
+				Evnt->KeyRelease( key, modstate );
 			break;
 
 		case SDL_KEYDOWN:
-			if ((event.key.keysym.sym == SDLK_SPACE) && GetModState(event.key.keysym.mod) & GEM_MOD_CTRL) {
+			if ((event.key.keysym.sym == SDLK_SPACE) && modstate & GEM_MOD_CTRL) {
 				core->PopupConsole();
 				break;
 			}
@@ -311,11 +318,11 @@ int SDLVideoDriver::SwapBuffers(void)
 				case SDLK_KP_ENTER:
 					key = GEM_RETURN;
 					break;
-				case SDLK_LSHIFT:
-				case SDLK_RSHIFT:
+				case SDLK_LALT:
+				case SDLK_RALT:
 					key = GEM_ALT;
 					break;
-				case SDLK_9:
+				case SDLK_TAB:
 					key = GEM_TAB;
 					break;
 				case SDLK_PAGEUP:
@@ -336,9 +343,9 @@ int SDLVideoDriver::SwapBuffers(void)
 					Evnt->OnSpecialKeyPress( key );
 			} else if (( key != 0 )) {
 				if (ConsolePopped)
-					core->console->OnKeyPress( key, GetModState(event.key.keysym.mod) );
+					core->console->OnKeyPress( key, modstate);
 				else if (Evnt)
-					Evnt->KeyPress( key, GetModState(event.key.keysym.mod) );
+					Evnt->KeyPress( key, modstate);
 			}
 			break;
 		case SDL_MOUSEMOTION:
@@ -394,49 +401,6 @@ int SDLVideoDriver::SwapBuffers(void)
 			Evnt->MouseUp( x, y, 1 << ( 0 ), GetModState(SDL_GetModState()) );
 	}
 
-	SDL_BlitSurface( backBuf, NULL, disp, NULL );
-	if (fadeColor.a) {
-		SDL_SetAlpha( extra, SDL_SRCALPHA, fadeColor.a );
-		SDL_Rect src = {
-			0, 0, Viewport.w, Viewport.h
-		};
-		SDL_Rect dst = {
-			xCorr, yCorr, 0, 0
-		};
-		SDL_BlitSurface( extra, &src, disp, &dst );
-	}
-
-	if (Cursor[CursorIndex] && !(DisableMouse&MOUSE_DISABLED)) {
-		SDL_Surface* temp = backBuf;
-		backBuf = disp; // FIXME: UGLY HACK!
-		if (DisableMouse&MOUSE_GRAYED) {
-			//used for greyscale blitting, fadeColor is unused
-			BlitGameSprite(Cursor[CursorIndex], CursorPos.x, CursorPos.y, BLIT_GREY, fadeColor, NULL, NULL, NULL, true);
-		} else {
-			BlitSprite(Cursor[CursorIndex], CursorPos.x, CursorPos.y, true);
-		}
-		backBuf = temp;
-	}
-
-	//handle tooltips
-	unsigned int delay = core->TooltipDelay;
-	// The multiplication by 10 is there since the last, disabling slider position is the eleventh
-	if (!ConsolePopped && (delay<TOOLTIP_DELAY_FACTOR*10) ) {
-		GetTime( time );
-		/** Display tooltip if mouse is idle */
-		if (( time - lastMouseTime ) > delay) {
-			if (Evnt)
-				Evnt->MouseIdle( time - lastMouseTime );
-		}
-
-		/** This causes the tooltip to be rendered directly to display */
-		SDL_Surface* tmp = backBuf;
-		backBuf = disp; // FIXME: UGLY HACK!
-		core->DrawTooltip();
-		backBuf = tmp;
-	}
-
-	SDL_Flip( disp );
 	return ret;
 }
 
@@ -1596,7 +1560,7 @@ Sprite2D* SDLVideoDriver::GetScreenshot( Region r )
 	SDL_Rect src = {r.x, r.y, r.w, r.h};
 
 
-	SDL_Surface* surf = SDL_CreateRGBSurface( SDL_HWSURFACE, Width, Height, 24,
+	SDL_Surface* surf = SDL_CreateRGBSurface( SDL_SWSURFACE, Width, Height, 24,
 				0xFF0000, 0x00FF00, 0x0000FF, 0x000000 );
 	SDL_BlitSurface( backBuf, (r.w && r.h) ? &src : NULL, surf, NULL);
 	void* pixels = malloc( Width * Height * 3 );
@@ -1646,7 +1610,7 @@ void SDLVideoDriver::DrawRect(const Region& rgn, const Color& color, bool fill,
 			long val = SDL_MapRGBA( backBuf->format, color.r, color.g, color.b, color.a );
 			SDL_FillRect( backBuf, &drect, val );
 		} else {
-			SDL_Surface * rectsurf = SDL_CreateRGBSurface( SDL_HWSURFACE | SDL_SRCALPHA, rgn.w, rgn.h, 8, 0, 0, 0, 0 );
+			SDL_Surface * rectsurf = SDL_CreateRGBSurface( SDL_SWSURFACE | SDL_SRCALPHA, rgn.w, rgn.h, 8, 0, 0, 0, 0 );
 			SDL_Color c;
 			c.r = color.r;
 			c.b = color.b;
@@ -1682,7 +1646,7 @@ void SDLVideoDriver::DrawRectSprite(const Region& rgn, const Color& color, const
 		long val = SDL_MapRGBA( surf->format, color.r, color.g, color.b, color.a );
 		SDL_FillRect( surf, &drect, val );
 	} else {
-		SDL_Surface * rectsurf = SDL_CreateRGBSurface( SDL_HWSURFACE | SDL_SRCALPHA, rgn.w, rgn.h, 8, 0, 0, 0, 0 );
+		SDL_Surface * rectsurf = SDL_CreateRGBSurface( SDL_SWSURFACE | SDL_SRCALPHA, rgn.w, rgn.h, 8, 0, 0, 0, 0 );
 		SDL_Color c;
 		c.r = color.r;
 		c.b = color.b;
@@ -1774,16 +1738,16 @@ void SDLVideoDriver::GetPixel(short x, short y, Color& c)
 
 void SDLVideoDriver::GetPixel(void *vptr, unsigned short x, unsigned short y, Color &c)
 {
-        SDL_Surface *surf = (SDL_Surface*)(vptr);
+	SDL_Surface *surf = (SDL_Surface*)(vptr);
 
-        SDL_LockSurface( surf );
-        unsigned char * pixels = ( ( unsigned char * ) surf->pixels ) +
-                ( ( y * surf->w + x) * surf->format->BytesPerPixel );
-        long val = 0;
-        ReadPixel(val, pixels, surf->format->BytesPerPixel);
-        SDL_UnlockSurface( surf );
+	SDL_LockSurface( surf );
+	unsigned char * pixels = ( ( unsigned char * ) surf->pixels ) +
+		( ( y * surf->w + x) * surf->format->BytesPerPixel );
+	long val = 0;
+	ReadPixel(val, pixels, surf->format->BytesPerPixel);
+	SDL_UnlockSurface( surf );
 
-        SDL_GetRGBA( val, surf->format, (Uint8 *) &c.r, (Uint8 *) &c.g, (Uint8 *) &c.b, (Uint8 *) &c.a );
+	SDL_GetRGBA( val, surf->format, (Uint8 *) &c.r, (Uint8 *) &c.g, (Uint8 *) &c.b, (Uint8 *) &c.a );
 }
 
 long SDLVideoDriver::GetPixel(void *vptr, unsigned short x, unsigned short y)
@@ -2396,13 +2360,14 @@ void SDLVideoDriver::ClickMouse(unsigned int button)
 
 void SDLVideoDriver::MouseClickEvent(Uint8 type, Uint8 button)
 {
-	SDL_MouseButtonEvent *event = new SDL_MouseButtonEvent();
+	SDL_Event *event = new SDL_Event();
 	event->type = type;
-	event->button = button;
-	event->state = (type==SDL_MOUSEBUTTONDOWN)?SDL_PRESSED:SDL_RELEASED;
-	event->x = CursorPos.x;
-	event->y = CursorPos.y;
-	SDL_PushEvent((SDL_Event *) event);
+	event->button.type = type;
+	event->button.button = button;
+	event->button.state = (type==SDL_MOUSEBUTTONDOWN)?SDL_PRESSED:SDL_RELEASED;
+	event->button.x = CursorPos.x;
+	event->button.y = CursorPos.y;
+	SDL_PushEvent(event);
 }
 
 void SDLVideoDriver::InitMovieScreen(int &w, int &h, bool yuv)
@@ -2426,6 +2391,12 @@ void SDLVideoDriver::InitMovieScreen(int &w, int &h, bool yuv)
 	subtitleregion.h = h/4;
 	subtitleregion.x = 0;
 	subtitleregion.y = h-h/4;
+
+	//same for SDL
+	subtitleregion_sdl.w = w;
+	subtitleregion_sdl.h = h/4;
+	subtitleregion_sdl.x = 0;
+	subtitleregion_sdl.y = h-h/4;
 }
 
 void SDLVideoDriver::showFrame(unsigned char* buf, unsigned int bufw,
@@ -2463,6 +2434,7 @@ void SDLVideoDriver::showFrame(unsigned char* buf, unsigned int bufw,
 	destRect.w = w;
 	destRect.h = h;
 
+	SDL_FillRect(disp, &subtitleregion_sdl, 0);
 	SDL_BlitSurface( sprite, &srcRect, disp, &destRect );
 	if (titleref>0)
 		DrawMovieSubtitle( titleref );
@@ -2499,6 +2471,7 @@ void SDLVideoDriver::showYUVFrame(unsigned char** buf, unsigned int *strides,
 	destRect.y = dsty;
 	destRect.w = w;
 	destRect.h = h;
+	SDL_FillRect(disp, &subtitleregion_sdl, 0);
 	SDL_DisplayYUVOverlay(overlay, &destRect);
 	if (titleref>0)
 		DrawMovieSubtitle( titleref );
@@ -2521,7 +2494,7 @@ int SDLVideoDriver::PollMovieEvents()
 					case SDLK_q:
 						return 1;
 					case SDLK_f:
-						SDL_WM_ToggleFullScreen( disp );
+						ToggleFullscreenMode(-1);
 						break;
 					default:
 						break;
@@ -2545,14 +2518,18 @@ void SDLVideoDriver::DrawMovieSubtitle(ieDword strRef)
 {
 	if (strRef!=subtitlestrref) {
 		core->FreeString(subtitletext);
+		if (!strRef)
+			return;
 		subtitletext = core->GetString(strRef);
 		subtitlestrref = strRef;
-		printf("Fetched subtitle %s\n", subtitletext);
 	}
-	if (subtitlefont) {
+	if (subtitlefont && subtitletext) {
 		// FIXME: ugly hack!
 		SDL_Surface* temp = backBuf;
 		backBuf = disp;
+		
+		//FYI: that 0 is pitch black
+		//SDL_FillRect(disp, &subtitleregion_sdl, 0);
 		subtitlefont->Print(subtitleregion, (unsigned char *) subtitletext, subtitlepal, IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_BOTTOM, true);
 		backBuf = temp;
 	}
