@@ -18,6 +18,14 @@
  *
  */
 
+#if defined(__HAIKU__)
+#include <unistd.h>
+#endif
+
+#ifdef ANDROID
+#include "swab.h"
+#endif
+
 #include "WEDImporter.h"
 
 #include "win32def.h"
@@ -100,7 +108,8 @@ int WEDImporter::AddOverlay(TileMap *tm, Overlay *overlays, bool rain)
 
 	memcpy(res, overlays->TilesetResRef,sizeof(ieResRef));
 	if (rain) {
-		strncat(res,"R",8);
+		if (strlen(res) < 8)
+			strcat(res,"R");
 		//no rain tileset available, rolling back
 		if (!gamedata->Exists(res,IE_TIS_CLASS_ID)) {
 			memcpy(res, overlays->TilesetResRef,sizeof(ieResRef));
@@ -108,6 +117,10 @@ int WEDImporter::AddOverlay(TileMap *tm, Overlay *overlays, bool rain)
 	}
 	TileOverlay *over = new TileOverlay( overlays->Width, overlays->Height );
 	DataStream* tisfile = gamedata->GetResource(res, IE_TIS_CLASS_ID);
+	if (!tisfile) {
+		delete over;
+		return -1;
+	}
 	PluginHolder<TileSetMgr> tis(IE_TIS_CLASS_ID);
 	tis->Open( tisfile );
 	for (int y = 0; y < overlays->Height; y++) {
@@ -155,6 +168,7 @@ int WEDImporter::AddOverlay(TileMap *tm, Overlay *overlays, bool rain)
 TileMap* WEDImporter::GetTileMap(TileMap *tm)
 {
 	int usedoverlays;
+	bool freenew = false;
 
 	if (!overlays.size()) {
 		return NULL;
@@ -162,10 +176,20 @@ TileMap* WEDImporter::GetTileMap(TileMap *tm)
 
 	if (!tm) {
 		tm = new TileMap();
+		freenew = true;
 	}
 
 	usedoverlays = AddOverlay(tm, &overlays.at(0), false);
-	AddOverlay(tm, &overlays.at(0), true);
+	if (usedoverlays == -1) {
+		if (freenew) {
+			delete tm;
+		}
+		return NULL;
+	}
+	// rain_overlays[0] is never used
+	// XXX: should fix AddOverlay not to load an overlay twice if there's no rain version!!
+	//AddOverlay(tm, &overlays.at(0), true);
+	tm->AddRainOverlay( NULL );
 
 	//reading additional overlays
 	int mask=2;
